'''
BSD Licence
Copyright (c) 2015, Science & Technology Facilities Council (STFC)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
    * Neither the name of the Science & Technology Facilities Council (STFC)
        nor the names of its contributors may be used to endorse or promote
        products derived from this software without specific prior written
        permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


Contents:
This module contains helper functions for the views package.

'''
from django.http import HttpResponseRedirect

from djcharme.node.constants import FORMAT_MAP
from djcharme.settings import LOGIN_REDIRECT_URL, NODE_URI, REDIRECT_FIELD_NAME
from django.utils.http import is_safe_url
from django.shortcuts import resolve_url


FORMAT = 'format'
DEPTH = 'depth'


def isGET(request):
    return request.method == 'GET'


def isPUT(request):
    return request.method == 'PUT'


def isDELETE(request):
    return request.method == 'DELETE'


def isPOST(request):
    return request.method == 'POST'


def isOPTIONS(request):
    return request.method == 'OPTIONS'


def isHEAD(request):
    return request.method == 'HEAD'


def isPATCH(request):
    return request.method == 'PATCH'


def content_type(request):
    content_type_ = request.environ.get('CONTENT_TYPE', None)
    if content_type_ is None:
        return None
    else:
        return content_type_.split(';')[0]


def get_format(request):
    try:
        return request.GET[FORMAT]
    except KeyError:
        return None


def get_depth(request):
    depth = request.GET.get(DEPTH)
    if depth is not None:
        try:
            return int(depth)
        except ValueError:
            return None
    return None


def http_accept(request):
    accept = request.META.get('HTTP_ACCEPT', None)
    if accept is None:
        return None
    return accept.split(';')[0].split(',')


def check_mime_format(mimeformat):
    """
    Map input MIME format to one of the accepted formats available.

    """
    # Set a default MIME format if none was set
    if mimeformat is None:
        mimeformat = 'application/ld+json'

    if '/' in mimeformat:
        for k, value in FORMAT_MAP.iteritems():
            if value in mimeformat:
                return k
    else:
        for k, value in FORMAT_MAP.iteritems():
            if k in mimeformat:
                return k


def get_extra_context(kwargs):
    """ Get any extra context from the keywords.
    """
    try:
        extra_context = kwargs['extra_context']
    except KeyError:
        extra_context = None
    return extra_context


def get_safe_redirect(request):
    """ Check that the URL is safe, i.e. it points back to whence it came.
    """
    redirect_to = request.REQUEST.get(REDIRECT_FIELD_NAME, '')
    # Ensure the user-originating redirection url is safe.
    if not is_safe_url(url=redirect_to, host=request.get_host()):
        if 'HTTP_REFERER' in request.META.keys():
            url = '%s/page' % NODE_URI
            if url in request.META['HTTP_REFERER']:
                return request.META['HTTP_REFERER'].split(NODE_URI)[1]
        redirect_to = resolve_url(LOGIN_REDIRECT_URL)
    return redirect_to


def not_authenticated(func):
    """ decorator that redirect user to next page if
    he is already logged.
    """
    def decorated(request, *args, **kwargs):
        if request.user.is_authenticated():
            next_ = request.GET.get(REDIRECT_FIELD_NAME,
                                    LOGIN_REDIRECT_URL)
            return HttpResponseRedirect(next_)
        return func(request, *args, **kwargs)
    return decorated


def validate_mime_format(request):
    req_format = [get_format(request)]
    if req_format[0] is None:
        req_format = http_accept(request)

    for mimeformat in req_format:
        ret = check_mime_format(mimeformat)
        if ret is not None:
            return ret
    return None




from django import forms
from django.contrib.auth import get_user_model
from django.contrib.sites.models import get_current_site
from django.template import loader
from django.utils.encoding import force_bytes
from django.utils.http import urlsafe_base64_encode
from django.utils.translation import ugettext_lazy as _


class UsernameReminderForm(forms.Form):
    email = forms.EmailField(label=_("Email"), max_length=254)

    def save(self, domain_override=None,
             subject_template_name='registration/username_reminder_subject.txt',
             email_template_name='registration/username_reminder_email.html',
             use_https=False,
             from_email=None, request=None):
        from django.core.mail import send_mail
        UserModel = get_user_model()
        email = self.cleaned_data["email"]
        active_users = UserModel._default_manager.filter(
            email__iexact=email, is_active=True)
        for user in active_users:
            # Make sure that no email is sent to a user that actually has
            # a password marked as unusable
            if not user.has_usable_password():
                continue
            if not domain_override:
                current_site = get_current_site(request)
                site_name = current_site.name
                domain = current_site.domain
            else:
                site_name = domain = domain_override
            c = {
                'email': user.email,
                'domain': domain,
                'site_name': site_name,
                'uid': urlsafe_base64_encode(force_bytes(user.pk)),
                'user': user,
                'protocol': 'https' if use_https else 'http',
            }
            subject = loader.render_to_string(subject_template_name, c)
            # Email subject *must not* contain newlines
            subject = ''.join(subject.splitlines())
            email = loader.render_to_string(email_template_name, c)
            send_mail(subject, email, from_email, [user.email])





from django.conf.urls import patterns, include, url
from django.contrib import admin
from django.contrib.auth import views as auth_views
from django_authopenid import views as oid_views

from djcharme.charme_security_model import LoginForm
from djcharme.views import node_gate, compose, endpoint, main_gui, search, \
    registration, resource, facets, auth


admin.autodiscover()

# Uncomment the next two lines to enable the admin:
# from django.contrib import admin
# admin.autodiscover()

IFORMAT = ["atom"]
IFORMATS_RE = '(' + '|'.join(IFORMAT) + ')'

urlpatterns = patterns('',
    # Must be use HTTPS
    #-----------------------------------------------------------
    (r'^admin/', include(admin.site.urls)),
    url(r'^oauth2/', include('provider.oauth2.urls', namespace='oauth2')),

    # ACCOUNTS
    #-----------------------------------------------------------
    # Registation
    url(r'^accounts/registration/$', registration.Registration.as_view(),
        name='registration'),
    # Login
    url(r'^accounts/login/$', auth.Login.as_view(), name='login'),
    # Profile
    url(r'^accounts/profile/$', registration.Profile.as_view(),
        name='profile'),
    # Profile change
    url(r'^accounts/profile/change/$', registration.profile_change,
        name='profile_change'),
    # Password change
    url(r'^accounts/password/change/$', auth_views.password_change,
        name='password_change'),
    url(r'^accounts/password/change/done/$', auth_views.password_change_done,
        name='password_change_done'),
    # Password reset
    url(r'^accounts/password/reset/$', auth_views.password_reset,
        {'post_reset_redirect' : '/accounts/password/reset/done/'},
        name="password_reset"),
    url(r'^accounts/password/reset/done/$', auth_views.password_reset_done),
    url(r'^accounts/password/reset/confirm/(?P<uidb64>[0-9A-Za-z]+)-(?P<token>.+)/$',
        auth_views.password_reset_confirm,
        {'post_reset_redirect' : '/accounts/password/reset/complete/'},
        name='password_reset_confirm'),
    url(r'^accounts/password/reset/complete/$',
        auth_views.password_reset_complete),
    # Username reminder
    url(r'^accounts/username/reminder/$', registration.username_reminder,
        name='username_reminder'),
    url(r'^accounts/username/reminder/done$',
        registration.username_reminder_done,
        name='username_reminder_done'),

    # replacement for django_authopenid register
    url(r'^accounts/register/$', auth.Register.as_view(),
        name='user_register'),
    # update for django_authopenid signin complete
    url(r'^accounts/signin/complete/$', oid_views.complete_signin,
        {'auth_form':LoginForm, 'on_success':auth.signin_success,
         'on_failure':auth.signin_failure},
        name='user_complete_signin'),
    # django_authopenid
    url(r'^accounts/', include('django_authopenid.urls')),

    #-----------------------------------------------------------

    # API - Add new annotation
    url(r'^insert/annotation', node_gate.insert, name='node_gate.insert'),

    # API - Modify an annotation
    url(r'^modify/annotation', node_gate.modify, name='node_gate.modify'),

    # API - Annotation status management
    url(r'^advance_status/', node_gate.advance_status,
        name='advance_status'),

    # API - Follow resources                     
    url(r'^user/following/(?P<resource_uri>.*)', node_gate.Following.as_view(),
        name='follow_resource'),
             
    # API - Report annotations
    url(r'^resource/(?P<resource_id>\w+)/reporttomoderator/$',
        node_gate.ReportToModerator.as_view(), name='report_resource'),

    # Display linked data resources
    # API
    url(r'^resource/(?P<resource_id>\w+)', node_gate.Resource.as_view(),
        name='process_resource'),
    # API
    url(r'^data/(?P<resource_id>\w+)', node_gate.ResourceData.as_view(),
        name='process_data'),
    # API/GUI
    url(r'^page/(?P<resource_id>\w+)', node_gate.ResourcePage.as_view(),
        name='process_page'),
    url(r'^annotation/$', resource.annotation, name='annotation'),
    url(r'^activity/$', resource.activity, name='activity'),

    # Annotation composition
    url(r'^compose/annotation', compose.compose_annotation,
        name='compose_annotation'),

    # HTTP SPARQL implementation
    url(r'^endpoint', endpoint.endpoint, name='endpoint'),

    # Opensearch
    url(r'^search/description', search.get_description,
        name='os_description'),
    url(r'^search/' + IFORMATS_RE, search.do_search, name='os_search'),
    url(r'^suggest/description', search.get_description,
        name='os_description'),
    url(r'^suggest/' + IFORMATS_RE, search.do_suggest, name='os_search'),

    # Tokens
    url(r'^token/validate/(?P<token>\w+)/(?P<expire>\w+)',
        registration.validate_token, name='validate_token'),
    url(r'^token/validate', registration.validate_token,
        name='validate_token'),
    url(r'^token/test', registration.test_token, name='test_token'),
    url(r'^token/response', registration.token_response,
        name='token_response'),
    url(r'^token/userinfo', registration.userinfo, name='userinfo'),

    # Facets
    url(r'^facets/test', facets.test_facets, name='test_facets'),

    # API - Server version
    url(r'^version', node_gate.version),

    # API - Vocab
    url(r'^vocab', node_gate.vocab),

    # Index pages
    url(r'^index/(\w+)', node_gate.index, name='charme.index.id'),
    url(r'^index', node_gate.index, name='index'),

    # GUI - Conditions of use
    url(r'^conditionsofuse/$', main_gui.conditions_of_use),

    # GUI - Following resources
    url(r'^following/$', main_gui.Following.as_view(), name='following-list'),
    url(r'^following/add$', main_gui.FollowingCreate.as_view(),
        name='following-add'),
    url(r'^following/(?P<pk>[0-9]+)/delete/$',
        main_gui.FollowingDelete.as_view(), name='following-delete'),
    
    # Anything else
    url(r'^', main_gui.welcome, name='charme.welcome'),

)



'''
BSD Licence
Copyright (c) 2014, Science & Technology Facilities Council (STFC)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
    * Neither the name of the Science & Technology Facilities Council (STFC)
        nor the names of its contributors may be used to endorse or promote
        products derived from this software without specific prior written
        permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Created on 4 May 2012

@author: Maurizio Nagni
'''
from ceda_markup.markup import createMarkup
from ceda_markup.opensearch.os_engine import OSEngine
from ceda_markup.opensearch.os_engine_helper import OSEngineHelper
from ceda_markup.opensearch.os_request import OS_PREFIX, OS_NAMESPACE
from ceda_markup.opensearch.os_request import OpenSearchDescription

from djcharme.opensearch.cimpl import COSQuery, COSAtomResponse


def setUp():
    query = COSQuery()
    # responses = [COSRDFResponse(), COSTurtleResponse(), COSJsonLDResponse(),
    # COSAtomResponse()]
    responses = [COSAtomResponse()]
    os_short_name = "CHARMe Search"
    os_description = "Use CHARMe Search to search for annotations"
    os_tags = "bodyType citingType dataType domainOfInterest motivation " \
        "organization region"
    os_adult_content = "false"
    os = OpenSearchDescription(os_short_name, os_description, os_tags=os_tags,
                               os_adult_content=os_adult_content)
    return OSEngine(query, responses, os, os_engine_helper=helper())


class helper(OSEngineHelper):
    """
    An implementation of the OSEngineHelper class used to provide additional
    information for the description document.

    """

    def __init__(self):
        """
        Constructor
        """
        super(OSEngineHelper)

    def additional_description(self, req_doc):
        """
        Overriding the OSEngineHelper method to add further tags into reqDoc.

        Args:
            req_doc: a request OpenSource document

        """
        markup = createMarkup('Query', OS_PREFIX, OS_NAMESPACE, req_doc)
        markup.set("role", "example")
        markup.set("status", "submitted")
        req_doc.append(markup)
        return req_doc




'''
BSD Licence
Copyright (c) 2014, Science & Technology Facilities Council (STFC)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
    * Neither the name of the Science & Technology Facilities Council (STFC)
        nor the names of its contributors may be used to endorse or promote
        products derived from this software without specific prior written
        permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Created on 2 Nov 2012

@author: mnagni
'''
import logging
import re

from django.conf import settings
from django.contrib.auth.models import AnonymousUser
from django.core.urlresolvers import reverse
from django.http.response import HttpResponse
from provider.oauth2.models import AccessToken


DJ_SECURITY_SHAREDSECRET_ERROR = 'No SECURITY_SHAREDSECRET parameter \
is defined in the application settings.py file. \
Please define it accordingly to the used LOGIN_SERVICE'

AUTHENTICATION_COOKIE_MISSING = 'The expected cookie is missing. \
Redirect to the authentication service'

DJ_MIDDLEWARE_IP_ERROR = 'No DJ_MIDDLEWARE_IP parameter \
is defined in the application settings.py file. \
Please define it accordingly to the machine/proxy seen by the LOGIN_SERVICE'

LOGIN_SERVICE_ERROR = 'No LOGIN_SETTING parameter is defined in the \
application settings.py file. Please define a proper URL to the \
authenticating service'

LOGOUT = 'logout'
LOGIN = 'login'

LOGGER = logging.getLogger(__name__)


def get_login_service_url():
    return reverse('login')


auth_tkt_name = lambda: getattr(settings, 'AUTH_TKT_NAME', 'auth_tkt')
token_field_name = lambda: getattr(settings, 'TOKEN_FIELD_NAME', 't')
security_filter = lambda: getattr(settings, 'SECURITY_FILTER', [])
redirect_field_name = lambda: getattr(settings, 'REDIRECT_FIELD_NAME', 'r')


def preapare_user_for_session(request, timestamp, userid, tokens, user_data):
    request.authenticated_user = {
        'timestamp': timestamp,
        'userid': userid,
        'tokens': tokens,
        'user_data': user_data
    }
    LOGGER.debug("preapare_user_for_session - Stored in request - " \
                 "userid:%s, user_data:%s", userid, user_data)
    request.session['accountid'] = userid


def filter_request(request, filters):
    """
        Checks a given strings against a list of strings.
        ** string ** string a url
        ** filters ** a list of strings
    """
    if filters is None:
        return False

    for ifilter in filters:
        if re.search(ifilter[0], request.path) and request.method in ifilter[1]:
            return True

    return False


def is_public_url(request):
    '''Test a given is public or secured - True if public'''
    url_filters = security_filter()

    # adds a default filter for reset password request
    reset_regexpr = '%s=[a-f0-9-]*$' % (token_field_name())
    if reset_regexpr not in url_filters:
        url_filters.append(reset_regexpr)

    if filter_request(request, url_filters):
        LOGGER.debug('is_public_url  -Public path and method %r / %r',
                     request.path, request.method)
        return True

    LOGGER.debug('is_public_url - Secured path and method %r / %r',
                 request.path, request.method)
    return False


def is_valid_token(token):
    if token:
        try:
            access_t = AccessToken.objects.get(token=token)
            if access_t.get_expire_delta() > 0:
                return True
        except AccessToken.DoesNotExist:
            return False
    return False


def _get_user(token):
    """
    Get the user information for the given token.

    Args:
        token (str): The auth token.

    Returns:
        User. Details about the user.

    """
    if token:
        try:
            access_t = AccessToken.objects.get(token=token)
        except AccessToken.DoesNotExist:
            LOGGER.warn("_get_user - Cannot get 'User' from access token")
            return None
    return access_t.user


def _get_client(token):
    """
    Get the client information for the given token.

    Args:
        token (str): The auth token.

    Returns:
        Client. Details about the client.

    """
    if token:
        try:
            access_t = AccessToken.objects.get(token=token)
        except AccessToken.DoesNotExist:
            LOGGER.warn("_get_client - Cannot get 'Client' from access token")
            return None
    return access_t.client


class SecurityMiddleware(object):
    """
        Validates if the actual user is authenticated against a
        given authentication service.
        Actually the middleware intercepts all the requests submitted
        to the underlying Django application and verifies if the presence
        or not of a valid paste cookie in the request.
    """

    def process_request(self, request):
        LOGGER.info('process_request - Request: %r',
                    request.build_absolute_uri())

        # The required URL is public
        if is_public_url(request):
            LOGGER.debug('process_request - URL is public')
            return

        # The request has an Access Token
        if request.environ.get('HTTP_AUTHORIZATION', None):
            for term in request.environ.get('HTTP_AUTHORIZATION').split():
                if is_valid_token(term):
                    request.user = _get_user(term)
                    request.client = _get_client(term)
                    LOGGER.debug('process_request - ' \
                    'Request has an access token')
                    return

        login_service_url = get_login_service_url()

        # An anonymous user want to access restricted resources
        if request.path != login_service_url \
            and isinstance(request.user, AnonymousUser):
            LOGGER.debug('process_request - Redirect to login page')
            return HttpResponse('<meta http-equiv="refresh" content="0; url='
                                + login_service_url + '" />', status=401)

        # An anonymous user wants to login
        if request.path == get_login_service_url():
            LOGGER.debug('process_request - Request is for login page')
            return

        LOGGER.debug('process_request - End of method')

    def process_response(self, request, response):
        if hasattr(response, 'url') and "access_token=" in response.url \
        and "token_type" in response.url:
            try:
                response.delete_cookie("sessionid")
                response.delete_cookie("csrftoken")
                # att_token = {'token': json.loads(response.content)}
                # return mm_render_to_response(request, att_token,
                # "token_response.html")
            except Exception as ex:
                LOGGER.warn('process_response - Caught: ' + ex)
        return response



'''
BSD Licence
Copyright (c) 2014, Science & Technology Facilities Council (STFC)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
    * Neither the name of the Science & Technology Facilities Council (STFC)
        nor the names of its contributors may be used to endorse or promote
        products derived from this software without specific prior written
        permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Created on 26 July 2013

@author: mnagni
'''


class CharmeError(Exception):
    """
        Raises all the exceptions related to the dj_charme
        class operations
    """

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value


class SerializeError(CharmeError):
    """
        General error during graph serialisation
    """

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value


class StoreConnectionError(CharmeError):
    """
        General connection error to the triplestore
    """

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value


class SecurityError(CharmeError):
    """
        General security error
    """

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value


class PasswordNotMachesError(CharmeError):
    """
        Throw when a user is not authenticated
    """

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value


class UserNotFoundError(CharmeError):
    """
        Throw when a user is not found in db
    """

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value


class MissingCookieError(SecurityError):
    """
        Missing cookie error
    """

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value


class ParseError(SecurityError):
    """
        Parse error
    """

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value


class NotFoundError(CharmeError):
    """
        Throw when an item is not found
    """

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value


class UserError(CharmeError):
    """
        Throw when a user creates an error
    """

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value





import sys
import site
import os

# this is the path to the virtualenv python site-package dir
vepath = 'VEPATH_PAR'
# this is the path to dir parent to the django project 
djangoPath = 'DJANGO_PATH'

# this is the project's name
projectLibPath = 'PROJECT_LIB_PATH'

ALLDIRS = [vepath, projectLibPath, djangoPath]

# Remember original sys.path.
prev_sys_path = list(sys.path)

# Add each new site-packages directory.
for directory in ALLDIRS:
    site.addsitedir(directory)

# Reorder sys.path so new directories at the front.
new_sys_path = []
for item in list(sys.path):
    if item not in prev_sys_path:
        new_sys_path.append(item)
        sys.path.remove(item)
sys.path[:0] = new_sys_path

os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'

from django.core.wsgi import get_wsgi_application
application = get_wsgi_application()




# Django settings for djcharme project.

########################
# MAIN DJANGO SETTINGS #
########################

# People who get code error notifications.
# In the format (('Full Name', 'email@example.com'),
#                ('Full Name', 'anotheremail@example.com'))
ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)
MANAGERS = ADMINS

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# On Unix systems, a value of None will cause Django to use the same
# timezone as the operating system.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'Europe/London'

# If you set this to True, Django will use timezone-aware datetimes.
USE_TZ = True

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-gb'

# A boolean that turns on/off debug mode. When set to ``True``, stack traces
# are displayed for error pages. Should always be set to ``False`` in
# production. Best set to ``True`` in local_settings.py
DEBUG = False

# Whether a user's session cookie expires when the Web browser is closed.
SESSION_EXPIRE_AT_BROWSER_CLOSE = True

SITE_ID = 1

SITE_PREFIX = ''

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = False

# Make this unique, and don't share it with anybody.
SECRET_KEY = "26192b77-eb5c-4b91-8f38-dad220fbe2c0ce26b7cb-5630-4e46-9204-d5b4484d76af72a57e13-a680-421f-89af-0df2f355e1fb"

# Tuple of IP addresses, as strings, that:
#   * See debug comments, when DEBUG is true
#   * Receive x-headers
INTERNAL_IPS = ("127.0.0.1",)

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    "django.template.loaders.filesystem.Loader",
    "django.template.loaders.app_directories.Loader",
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
)

#########
# PATHS #
#########

import os


# Full filesystem path to the project.
PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))

# Name of the directory for the project.
PROJECT_DIRNAME = PROJECT_ROOT.split(os.sep)[-1]

# Every cache key will get prefixed with this value - here we set it to
# the name of the directory the project is in to try and use something
# project specific.
CACHE_MIDDLEWARE_KEY_PREFIX = PROJECT_DIRNAME

# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = "/static/"

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = os.path.join(PROJECT_ROOT, STATIC_URL.strip("/"))

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://media.lawrence.com/media/", "http://example.com/media/"
MEDIA_URL = STATIC_URL + "media/"

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/home/media/media.lawrence.com/media/"
MEDIA_ROOT = os.path.join(PROJECT_ROOT, *MEDIA_URL.strip("/").split("/"))

# Package/module name to import the root urlpatterns from for the project.
ROOT_URLCONF = "%s.urls" % PROJECT_DIRNAME

# Put strings here, like "/home/html/django_templates"
# or "C:/www/django/templates".
# Always use forward slashes, even on Windows.
# Don't forget to use absolute paths, not relative paths.
TEMPLATE_DIRS = (os.path.join(PROJECT_ROOT, "templates"),)


################
# APPLICATIONS #
################
INSTALLED_APPS = (
    "cedatheme_mf54",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.redirects",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.sitemaps",
    "django.contrib.staticfiles",
    "djcharme",
    "cookielaw",
    'provider',
    'provider.oauth2',
    'registration',
    'django_authopenid',
)



# For Profiles
AUTH_PROFILE_MODULE = 'djcharme.userprofile.models.UserProfile'

# List of processors used by RequestContext to populate the context.
# Each one should be a callable that takes the request object as its
# only parameter and returns a dictionary to add to the context.
TEMPLATE_CONTEXT_PROCESSORS = (
    "django.contrib.auth.context_processors.auth",
    "django.contrib.messages.context_processors.messages",
    "django.core.context_processors.debug",
    "django.core.context_processors.i18n",
    "django.core.context_processors.static",
    "django.core.context_processors.media",
    "django.core.context_processors.request",
    "django.core.context_processors.tz",
    "django_authopenid.context_processors.authopenid",
)

# List of middleware classes to use. Order is important; in the request phase,
# these middleware classes will be applied in the order given, and in the
# response phase the middleware will be applied in reverse order.
MIDDLEWARE_CLASSES = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.redirects.middleware.RedirectFallbackMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "djcharme.security_middleware.SecurityMiddleware",
    "djcharme.charme_middleware.CharmeMiddleware",
    "django_authopenid.middleware.OpenIDMiddleware",
)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format':
            '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d ' \
            '%(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler'
        },
        'console':{
            'level':'DEBUG',
            'class':'logging.StreamHandler',
            'formatter': 'verbose'
        }
    },
    'loggers': {
        'django': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True,
        },
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
        'djcharme': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True,
        }
    }
}


###################
# DEPLOY SETTINGS #
###################

# These settings are used by the default fabfile.py provided.
# Check fabfile.py for defaults.

# FABRIC = {
#     "SSH_USER": "", # SSH username
#     "SSH_PASS":  "", # SSH password (consider key-based authentication)
#     "SSH_KEY_PATH":  "", # Local path to SSH key file, for key-based auth
#     "HOSTS": [], # List of hosts to deploy to
#     "VIRTUALENV_HOME":  "", # Absolute remote path for virtualenvs
#     "PROJECT_NAME": "", # Unique identifier for project
#     "REQUIREMENTS_PATH": "", # Path to pip requirements, relative to project
#     "GUNICORN_PORT": 8000, # Port gunicorn will listen on
#     "LOCALE": "en_US.UTF-8", # Should end with ".UTF-8"
#     "LIVE_HOSTNAME": "www.example.com", # Host for public site.
#     "REPO_URL": "", # Git or Mercurial remote repo URL for the project
#     "DB_PASS": "", # Live database password
#     "ADMIN_PASS": "", # Live admin user password
# }

# Default cedasite configuration parameters. May be overridden in the
# local_settings.py
ALLOWED_HOSTS = ['*']

PASSWORD_HASHERS = (
    'django.contrib.auth.hashers.BCryptSHA256PasswordHasher',
    'django.contrib.auth.hashers.BCryptPasswordHasher',
    'django.contrib.auth.hashers.PBKDF2PasswordHasher',
    'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
    'django.contrib.auth.hashers.SHA1PasswordHasher',
    'django.contrib.auth.hashers.MD5PasswordHasher',
    'django.contrib.auth.hashers.CryptPasswordHasher',
)

AUTHENTICATION_BACKENDS = (
    'djcharme.charme_security_model.CharmeAuthenticationBackend',
    'django.contrib.auth.backends.ModelBackend',
)

# Set header fields for HTTP OPTIONS method response.  If omitted, the code
# will default to the setting in charme_middleware.CharmeMiddleware
if "djcharme.charme_middleware.CharmeMiddleware" in MIDDLEWARE_CLASSES:
    OPTIONS_HDR_RESPONSE = {
        'Access-Control-Allow-Methods': 'GET, OPTIONS, PUT, POST, DELETE',
        'Access-Control-Allow-Headers': 'X-CSRFToken, X-Requested-With, ' \
            'x-requested-with, Content-Type, Content-Length, Authorization',
        'Access-Control-Max-Age': 10,
        'Content-Type': "text/plain"
    }

OAUTH_SCOPES = (
    (1, 'add_annotation'),
    (2, 'update_annotation'),
)


# Modify temporarily the session serializer because the json serializer in
# Django 1.6 can't serialize openid.yadis.manager.YadisServiceManager objects
SESSION_SERIALIZER = 'django.contrib.sessions.serializers.PickleSerializer'


# Sets URIs that are *NOT* secured.  Entries are URIs and HTTP method tuples
SECURITY_FILTER = [
    ("/.*", ("OPTIONS",)),  # Allow all OPTIONS requests
    ("/search", ("GET", "OPTIONS")),
    ("/suggest", ("GET", "OPTIONS")),
    ("/accounts/login", ("GET", "POST")),
    ("/accounts/password/reset", ("GET", "POST")),
    ("/accounts/username/reminder", ("GET", "POST")),
    ("/accounts/register", ("GET", "POST")),
    ("/accounts/registration", ("GET", "POST")),
    ("/accounts/signin/complete/", ("GET", "POST")),
    ("/conditionsofuse", ("GET")),
    ("/data", ("GET")),
    ("/index", ("GET")),
    ("/page", ("GET")),
    ("/resource", ("GET")),
    ("/oauth2\/?", ("GET", "POST")),
    ("admin", ("GET", "POST")),
    ("/token/validate/", ("GET",)),
    ("/token/test", ("GET",)),
    ("/version", ("GET",)),
    ("/vocab", ("GET",)),
]

LOGIN_REDIRECT_URL = '/accounts/profile/'
REDIRECT_FIELD_NAME = 'next'

# send an email when an open id is linked to a user
SEND_MAIL = True

# Email settings used when resetting passwords
EMAIL_HOST = 'localhost'
EMAIL_PORT = 1025
EMAIL_HOST_USER = ''
EMAIL_HOST_PASSWORD = ''
EMAIL_USE_TLS = False
DEFAULT_FROM_EMAIL = 'testing@127.0.0.1:8000'

# SESSION_COOKIE_SECURE = True
# CSRF_COOKIE_SECURE = True
##################
# LOCAL SETTINGS #
##################
# Allow any settings to be defined in local_settings.py which should be
# ignored in your version control system allowing for settings to be
# defined per machine.

try:
    from local_settings import *
except ImportError:
    pass



'''
BSD Licence
Copyright (c) 2015, Science & Technology Facilities Council (STFC)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
    * Neither the name of the Science & Technology Facilities Council (STFC)
        nor the names of its contributors may be used to endorse or promote
        products derived from this software without specific prior written
        permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'''
__version__ = '0.8.6'


import os

from django.conf import settings
from django.core.context_processors import csrf
from django.core.exceptions import ObjectDoesNotExist
from django.http.response import Http404
from django.shortcuts import render_to_response, render
from django.template.context import RequestContext


LOAD_SAMPLE = 'LOAD_SAMPLE'
HTTP_PROXY = 'HTTP_PROXY'
HTTP_PROXY_PORT = 'HTTP_PROXY_PORT'


def get_resource(file_name):
    return os.path.join(__path__[0], 'resources', file_name)


def mm_render_to_response(request, context, page_to_render):
    """
    Exploits a 'render_to_response' action. The advantage of this method
    is to contains a number of operations that are expected to be  called
    for each page rendering, for example passing the application version number

    **Parameters**
        * HttpRequest_ **request**
            a django HttpRequest instance
        * `dict` **context**
            a dictionary where to pass parameter to the rendering function
        * `string` **page_to_render**
            the html page to render
    """
    if context is None or not isinstance(context, dict):
        raise Exception("Cannot render an empty context")

    # context['version'] = assemble_version()
    context.update(csrf(request))
    rcontext = RequestContext(request, context)
    return render_to_response(page_to_render, rcontext)


def mm_render_to_response_error(request, page_to_render, status):
    """
    Exploits a 'render_to_response' action. The advantage of this method
    is to contains a number of operations that are expected to be  called
    for each page rendering, for example passing the application version number

    **Parameters**
        * HttpRequest_ **request**
            a django HttpRequest instance
        * `dict` **context**
            a dictionary where to pass parameter to the rendering function
        * `string` **page_to_render**
            the html page to render
    """
    context = {}
    context.update(csrf(request))
    rcontext = RequestContext(request, context)
    return render(request, page_to_render, context_instance=rcontext,
                  status=status)



"""
Admin stuff.

"""
from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.models import User
from djcharme.models import FollowedResource, UserProfile, Organization, \
    OrganizationClient, OrganizationUser
from provider.oauth2.admin import ClientAdmin
from provider.oauth2.models import Client


class FollowedResourceAdmin(admin.ModelAdmin):
    """
    The Followed Resource Admin.

    """
    fields = ['user', 'resource']
    list_display = ('user', 'resource')
    list_filter = ['user']

class UserProfileInline(admin.StackedInline):
    """
    The in-line display of the User Profile.

    """
    model = UserProfile
    can_delete = False
    verbose_name = 'Profile'
    verbose_name_plural = 'Profiles'
    list_display = ('show_email')


class OrganizationUserInline(admin.StackedInline):
    """
    The in-line display of the Organization User.

    """
    model = OrganizationUser
    can_delete = False
    verbose_name = 'Role'
    verbose_name_plural = 'Roles'
    list_display = ('organization', 'role')
    extra = 1


class OrganizationClientInline(admin.StackedInline):
    """
    The in-line display of the Organization Client.

    """
    model = OrganizationClient
    verbose_name = 'Organization'
    verbose_name_plural = 'Organization'
    extra = 1
    fk_name = "client"


class OrganizationAdmin(admin.ModelAdmin):
    """
    The Organization Admin.

    """
    list_display = ('name', 'primary_email')


class OrganizationClientAdmin(admin.ModelAdmin):
    """
    The Organization Client Admin.

    """
    list_display = ('organization', 'client')
    list_filter = ['organization']


class OrganizationUserAdmin(admin.ModelAdmin):
    """
    The Organization User Admin.

    """
    fields = ['organization', 'user', 'role']
    list_display = ('organization', 'user', 'role')
    list_filter = ['organization', 'user']


class ClientAdmin_(ClientAdmin):
    """
    The new version of ClientAdmin, which includes the Organization Client.

    """
    inlines = [OrganizationClientInline]
    list_display = ('url', 'get_organization', 'redirect_uri', 'client_id',
                    'client_type')

    def get_organization(self, obj):
        """
        Get the name of the organization, there should only be one.

        """
        return obj.organizationclient_set.first()

    get_organization.short_description = 'organization'


class UserAdmin_(UserAdmin):
    """
    The new version of UserAdmin, which includes the User Profile and
    Organization User.

    """
    inlines = [UserProfileInline, OrganizationUserInline]
    list_display = ('username', 'get_role', 'email', 'first_name', 'last_name',
                    'is_staff')

    def get_role(self, obj):
        """
        Get the role of the user, they may have more than one role.

        """
        admin_ = False
        user = False
        for organizationuser in obj.organizationuser_set.iterator():
            if organizationuser.role == 'admin':
                admin_ = True
            else:
                user = True

        if admin_ and user:
            return 'admin & user'
        elif admin_:
            return 'admin'
        elif admin_:
            return 'user'
        else:
            return ''

    get_role.short_description = 'roles in organizations'


# Re-register UserAdmin
admin.site.unregister(User)
admin.site.register(User, UserAdmin_)
admin.site.unregister(Client)
admin.site.register(Client, ClientAdmin_)
admin.site.register(FollowedResource, FollowedResourceAdmin)
admin.site.register(Organization, OrganizationAdmin)
admin.site.register(OrganizationUser, OrganizationUserAdmin)
admin.site.register(OrganizationClient, OrganizationClientAdmin)



'''
BSD Licence
Copyright (c) 2015, Science & Technology Facilities Council (STFC)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
    * Neither the name of the Science & Technology Facilities Council (STFC)
        nor the names of its contributors may be used to endorse or promote
        products derived from this software without specific prior written
        permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Created on 14 May 2013

@author: mnagni
'''
import logging

from django.http.response import HttpResponseRedirectBase, HttpResponse
from rdflib.graph import ConjunctiveGraph
from rdflib.term import URIRef

from djcharme.charme_middleware import CharmeMiddleware
from djcharme.node.actions import  insert_rdf, generate_graph, rdf_format_from_mime
from djcharme.views import isGET, isPOST, isPUT, isDELETE, isHEAD, isPATCH, \
    content_type


LOGGING = logging.getLogger(__name__)


class HttpResponseSeeOther(HttpResponseRedirectBase):
    status_code = 303


def endpoint(request):
    if isGET(request):
        return processGET(request)
    if isPUT(request):
        return processPUT(request)
    if isDELETE(request):
        return processDELETE(request)
    if isPOST(request):
        return processPOST(request)
    if isHEAD(request):
        return processHEAD(request)
    if isPATCH(request):
        return processPATCH(request)


def get_graph_from_request(request):
    graph = request.GET.get('graph', 'default')

    if graph == 'default':
        graph = None
    return graph


def processGET(request):
    '''
        Returns an httplib.HTTPRequest
    '''
    return processHEAD(request, True)


def processPUT(request):
    graph = get_graph_from_request(request)
    payload = request.body

    conjunctive_graph = None
    query_object = None
    if graph is None:
        conjunctive_graph = ConjunctiveGraph(store=CharmeMiddleware.get_store())
        query_object = '''
            DROP SILENT DEFAULT;
            '''
        query_object = ''
    else:
        conjunctive_graph = ConjunctiveGraph(store=CharmeMiddleware.get_store())
        query_object = '''
            DROP SILENT GRAPH <%s>;
            '''
        query_object = query_object % (graph)
    conjunctive_graph.update(query_object)
    insert_rdf(payload,
               content_type(request),
               request.user,
               request.client,
               graph)

    return HttpResponse(status=204)


def processDELETE(request):
    graph = get_graph_from_request(request)

    conjunctive_graph = None
    query_object = None
    if graph is None:
        conjunctive_graph = ConjunctiveGraph(store=CharmeMiddleware.get_store())
        query_object = '''
            DROP DEFAULT;
            '''
        query_object = ''
    else:
        conjunctive_graph = ConjunctiveGraph(store=CharmeMiddleware.get_store())
        query_object = '''
            DROP GRAPH <%s>;
            '''
        query_object = query_object % (graph)
    conjunctive_graph.update(query_object)

    return HttpResponse(status=204)


def processPOST(request):
    graph = get_graph_from_request(request)
    payload = request.body
    insert_rdf(payload,
               content_type(request),
               request.user,
               request.client,
               graph)

    return HttpResponse(status=204)


def processHEAD(request, return_content=False):
    '''
        Returns an httplib.HTTPRequest
    '''
    graph = get_graph_from_request(request)
    accept = _validate_mime_format(request, 'application/rdf+xml')

    if accept == None:
        return HttpResponse(status=406)

    conjunctive_graph = None
    if graph is None:
        conjunctive_graph = ConjunctiveGraph(store=CharmeMiddleware.get_store())
    else:
        conjunctive_graph = generate_graph(CharmeMiddleware.get_store(),
                                           URIRef(graph))

    content = conjunctive_graph.serialize(format=accept)

    if return_content:
        return HttpResponse(content=content)
    return HttpResponse()


def processPATCH(request):
    return HttpResponse(status=501)


def _validate_mime_format(request, default=None):
    """
    Returns the first valid mimetype as mapped as rdf format
    """
    req_formats = request.META.get('HTTP_ACCEPT', default)
    req_formats = req_formats.split(',')
    for req_format in req_formats:
        if rdf_format_from_mime(req_format) != None:
            return req_format
    if ((len(req_formats) == 0) or
        (len(req_formats) == 1 and req_formats[0] == '*/*')):
        return default
    return None



'''
BSD Licence
Copyright (c) 2015, Science & Technology Facilities Council (STFC)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
    * Neither the name of the Science & Technology Facilities Council (STFC)
        nor the names of its contributors may be used to endorse or promote
        products derived from this software without specific prior written
        permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


Contents:
This module contains views of the user pages, excluding login and registration.

Created on 19 Aug 2013

@author: mnagni
'''
from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.forms.forms import NON_FIELD_ERRORS
from django.shortcuts import render, redirect
from django.views.generic.edit import CreateView, DeleteView
from django.views.generic.list import ListView

from djcharme import mm_render_to_response
from djcharme.models import FollowedResource
from djcharme.node import is_following_resource, resource_exists


def welcome(request):
    """
    Display the welcome page.

    """
    context = {}
    return mm_render_to_response(request, context, 'welcome.html')


def conditions_of_use(request):
    """
    Display the conditions of use.

    """
    context = {}
    return mm_render_to_response(request, context, 'conditions_of_use.html')


class Following(ListView):
    """
    Display the list of resources that are being followed by the user.

    """

    def get(self, request):
        following = self.get_queryset()
        if len(following) == 0:
            return redirect(reverse('following-add'))
        context = {'followed_resource' : following}
        return render(request,
                      'followed-resources/followed_resource.html',
                      context)
    
    def get_queryset(self):
        """
        Return the resources followed by the user.

        """
        return FollowedResource.objects.filter(
            user=self.request.user
        ).order_by('resource')


class FollowingCreate(CreateView):
    """
    Add a resource to the list of resources that are being followed by the user.

    """
    model = FollowedResource
    fields = ['resource']
    template_name = 'followed-resources/followed_resource_form.html'
    success_url = reverse_lazy('following-list')
        
    def form_valid(self, form):
        """
        Validate the form. Check that the user is not already following the
        resource and that the resource exists in the triple store.
        
        """
        if is_following_resource(self.request.user, form.instance.resource.strip()):
            form.full_clean()
            form._errors[NON_FIELD_ERRORS] = form.error_class(['You are already following this resource'])
            form.non_field_errors()
            return super(FollowingCreate, self).form_invalid(form)            

        if not (resource_exists(form.instance.resource.strip())):
            form.full_clean()
            form._errors[NON_FIELD_ERRORS] = form.error_class(['Resource does not exist on this system'])
            form.non_field_errors()
            return super(FollowingCreate, self).form_invalid(form)

        form.instance.user = self.request.user
        return super(FollowingCreate, self).form_valid(form)

    
class FollowingDelete(DeleteView):
    """
    Delete the resource that is being followed by the user.

    """
    model = FollowedResource
    template_name = 'followed-resources/followed_resource_confirm_delete.html'   
    success_url = reverse_lazy('following-list')




'''
Created on 17 May 2013

@author: mnagni
'''

import logging

from djcharme import mm_render_to_response
from djcharme.node.actions import  insert_rdf
from djcharme.views import isPOST


LOGGING = logging.getLogger(__name__)

HTML_NS = "{http://www.w3.org/1999/xhtml}"

'''
def extract_alternative_links(link):
    h = httplib2.Http(".cache")
    resp, content = h.request(link, "GET", headers={'accept':'*/*'} )
    parser = html5lib.HTMLParser(tree=html5lib.getTreeBuilder("lxml"))
    doc = parser.parse(content)
    ret = []
    for el in doc.findall('%shead//%slink' % (HTML_NS, HTML_NS)):
        if hasattr(el, 'attrib') and el.attrib.has_key('rel') \
                and el.attrib['rel'] == "alternate":
            ret.append(el.get('type'))
    print ret
'''


def compose_annotation(request):
    context = {}
    logging.debug(request)

    if request.REQUEST.get('target_link', None):
        context['target_link'] = request.REQUEST.get('target_link')
        return mm_render_to_response(request, context,
                                     'compose_annotation.html')

    if isPOST(request):
        insert_rdf('', request.body, request.user, request.client)
    # target_link = request.REQUEST['target_link']
    # extract_alternative_links(target_link)
    return mm_render_to_response(request, context, 'compose_annotation.html')



'''
BSD Licence
Copyright (c) 2015, Science & Technology Facilities Council (STFC)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
    * Neither the name of the Science & Technology Facilities Council (STFC)
        nor the names of its contributors may be used to endorse or promote
        products derived from this software without specific prior written
        permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'''
import datetime
import logging

from django.forms.fields import CharField
from django.forms.forms import Form
from django.http.response import HttpResponseRedirect

from djcharme import mm_render_to_response
from djcharme.node.actions import change_annotation_state, is_update_allowed, \
    find_annotation_graph_name
from djcharme.node.constants import  CITO, CONTENT, DC, DCTERMS, FOAF, \
    INVALID, OA, PROV, RDF, RETIRED, SKOS


LOGGING = logging.getLogger(__name__)

def annotation(request, resource_uri=None, graph=None):
    LOGGING.debug('Annotation request received')
    context = {}

    if request.method == 'POST':
        LOGGING.debug('method is POST')
        return _delete(request)
    else:  # GET
        context['uri'] = resource_uri

        triples = graph.triples((resource_uri, PROV['wasInvalidatedBy'], None))
        for triple in triples:
            context['wasInvalidatedBy'] = triple[2]

        triples = graph.triples((resource_uri, PROV['wasGeneratedBy'], None))
        for triple in triples:
            context['wasGeneratedBy'] = triple[2]

        triples = graph.triples((resource_uri, PROV['wasRevisionOf'], None))
        for triple in triples:
            context['wasRevisionOf'] = triple[2]

        triples = graph.triples((None, DCTERMS['title'], None))
        for triple in triples:
            context['title'] = triple[2]

        triples = graph.triples((resource_uri, OA['annotatedAt'], None))
        for triple in triples:
            ann_at = datetime.datetime.strptime(triple[2],
                                                "%Y-%m-%dT%H:%M:%S.%f")
            ann_at = ann_at.strftime("%H:%M %d %b %Y")
            context['annotated_at'] = ann_at

        triples = graph.triples((resource_uri, OA['annotatedBy'], None))
        for triple in triples:
            prov_triples = graph.triples((triple[2], FOAF['name'], None))
            for a_triple in prov_triples:
                context['organization_name'] = a_triple[2]
                context['organization_uri'] = a_triple[0]

        triples = graph.triples((resource_uri, OA['serializedBy'], None))
        for triple in triples:
            prov_triples = graph.triples((triple[2], FOAF['name'], None))
            for a_triple in prov_triples:
                context['agent_name'] = a_triple[2]
                context['agent_uri'] = a_triple[0]

        triples = graph.triples((resource_uri, OA['serializedAt'], None))
        for triple in triples:
            ann_at = datetime.datetime.strptime(triple[2],
                                                "%Y-%m-%dT%H:%M:%S.%f")
            ann_at = ann_at.strftime("%H:%M %d %b %Y")
            context['serialized_at'] = ann_at

        triples = graph.triples((resource_uri, OA['motivatedBy'], None))
        motivations = []
        for triple in triples:
            motivations.append(triple[2])
        context['motivations'] = motivations

        # person
        triples = graph.triples((None, RDF['type'], FOAF['Person']))
        for triple in triples:
            context['person'] = triple[0]

        triples = graph.triples((None, FOAF['givenName'], None))
        for triple in triples:
            context['first_name'] = triple[2]

        triples = graph.triples((None, FOAF['familyName'], None))
        for triple in triples:
            context['last_name'] = triple[2]

        triples = graph.triples((None, FOAF['mbox'], None))
        for triple in triples:
            context['email'] = triple[2]

        triples = graph.triples((None, FOAF['accountName'], None))
        for triple in triples:
            context['username'] = triple[2]

        # body
        triples = graph.triples((resource_uri, OA['hasBody'], None))
        bodies = []
        for triple in triples:
            body_triples = graph.triples((triple[2], RDF['type'], None))
            types = []
            for b_triple in body_triples:
                types.append(b_triple[2])
            body_triples = graph.triples((triple[2], CONTENT['chars'], None))
            text = []
            for b_triple in body_triples:
                text.append(b_triple[2])
            bodies.append((triple[2], types, text))
        context['bodies'] = bodies

        # target
        triples = graph.triples((resource_uri, OA['hasTarget'], None))
        targets = []
        for triple in triples:
            target_triples = graph.triples((triple[2], RDF['type'], None))
            types = []
            for t_triple in target_triples:
                types.append(t_triple[2])
            target_triples = graph.triples((triple[2], CONTENT['chars'], None))
            text = []
            for t_triple in target_triples:
                text.append(t_triple[2])
            targets.append((triple[2], types, text))
        context['targets'] = targets

        # citation
        triples = graph.triples((None, CITO['hasCitingEntity'], None))
        for triple in triples:
            context['citing_entity'] = triple[2]

        triples = graph.triples((None, CITO['hasCitedEntity'], None))
        for triple in triples:
            context['cited_entity'] = triple[2]

        triples = graph.triples((None, CITO['hasCitationCharacterization'],
                                 None))
        for triple in triples:
            context['citation_characterization'] = triple[2]

        # delete button
        context['delete'] = False

        # message
        if request.user.username == None or request.user.username == '':
            context['message'] = 'If you are the author of this annotation, ' \
                'or a moderator for %s, you may delete this annotation by ' \
                'logging in' % context['organization_name']
        else:
            # delete button
            graph_name = find_annotation_graph_name(resource_uri)
            update_allowed = is_update_allowed(graph, resource_uri, request)
            if (graph_name != INVALID and graph_name != RETIRED
                and update_allowed):
                context['delete'] = True

        orig_values = {}
        orig_values['resource_uri'] = resource_uri
        resource_form = ResourceForm(initial=orig_values)
        context['resource_form'] = resource_form

        return mm_render_to_response(request, context, 'resources/annotation.html')


def annotation_index(request, tmp_g, graph_name):
    LOGGING.debug('Annotation index request received')
    context = {}
    context['graph_name'] = graph_name

    annotations = []
    for subject, _, _ in tmp_g.triples((None, None, OA['Annotation'])):
        annotations.append(subject)
    context['annotations'] = annotations
    return mm_render_to_response(request, context, 'index.html')


def activity(request, resource_uri, graph):
    LOGGING.debug('Activity request received')
    context = {}

    context['uri'] = resource_uri

    triples = graph.triples((resource_uri, PROV['invalidated'], None))
    for triple in triples:
        context['invalidated'] = triple[2]

    triples = graph.triples((resource_uri, PROV['generated'], None))
    for triple in triples:
        context['generated'] = triple[2]

    triples = graph.triples((resource_uri, PROV['wasEndedAt'], None))
    for triple in triples:
            at = datetime.datetime.strptime(triple[2], "%Y-%m-%dT%H:%M:%S.%f")
            at = at.strftime("%H:%M %d %b %Y")
            context['wasEndedAt'] = at

    triples = graph.triples((resource_uri, PROV['wasEndedBy'], None))
    for triple in triples:
        activity_triples = graph.triples((triple[2], FOAF['givenName'], None))
        for a_triple in activity_triples:
            context['first_name'] = a_triple[2]

        activity_triples = graph.triples((triple[2], FOAF['familyName'], None))
        for a_triple in activity_triples:
            context['last_name'] = a_triple[2]

        activity_triples = graph.triples((triple[2], FOAF['mbox'], None))
        for a_triple in activity_triples:
            context['email'] = a_triple[2]

    return mm_render_to_response(request, context, 'resources/activity.html')


def agent(request, resource_uri, graph):
    LOGGING.debug('Agent request received')
    context = {}

    context['uri'] = resource_uri

    triples = graph.triples((resource_uri, FOAF['name'], None))
    for triple in triples:
        context['name'] = triple[2]

    return mm_render_to_response(request, context, 'resources/agent.html')


def composite(request, resource_uri, graph):
    LOGGING.debug('Composite request received')
    context = {}

    context['uri'] = resource_uri

    triples = graph.triples((resource_uri, OA['item'], None))
    items = []
    for triple in triples:
        items.append(triple[2])
    context['items'] = items

    return mm_render_to_response(request, context, 'resources/composite.html')


def person(request, resource_uri, graph):
    LOGGING.debug('Person request received')
    context = {}

    context['uri'] = resource_uri

    triples = graph.triples((resource_uri, FOAF['givenName'], None))
    for triple in triples:
        context['first_name'] = triple[2]

    triples = graph.triples((resource_uri, FOAF['familyName'], None))
    for triple in triples:
        context['last_name'] = triple[2]

    triples = graph.triples((resource_uri, FOAF['mbox'], None))
    for triple in triples:
        context['email'] = triple[2]

    triples = graph.triples((None, FOAF['accountName'], None))
    for triple in triples:
        context['username'] = triple[2]

    return mm_render_to_response(request, context, 'resources/person.html')


def resource(request, resource_uri, graph):
    LOGGING.debug('Resource request received')
    context = {}

    context['uri'] = resource_uri

    triples = graph.triples((resource_uri, RDF['type'], None))
    types = []
    for triple in triples:
        types.append(triple[2])
    context['types'] = types

    triples = graph.triples((resource_uri, DC['format'], None))
    for triple in triples:
        context['format'] = triple[2]

    triples = graph.triples((resource_uri, CONTENT['chars'], None))
    text = []
    for triple in triples:
        text.append(triple[2])
    context['text'] = text

    triples = graph.triples((resource_uri, SKOS['prefLabel'], None))
    tags = []
    for triple in triples:
        tags.append(triple[2])
    context['tags'] = tags

    return mm_render_to_response(request, context, 'resources/resource.html')


def _delete(request):
    LOGGING.debug('Annotation delete request received')
    resource_uri = request.POST['resource_uri']
    change_annotation_state(resource_uri, RETIRED, request)
    return HttpResponseRedirect(resource_uri)


class ResourceForm(Form):
    resource_uri = CharField(max_length=100, required=True)





'''
BSD Licence
Copyright (c) 2015, Science & Technology Facilities Council (STFC)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
    * Neither the name of the Science & Technology Facilities Council (STFC)
        nor the names of its contributors may be used to endorse or promote
        products derived from this software without specific prior written
        permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


Contents: This module contains views of the login. If the login was via an
unknown OpenID then Register is used to link the OpenID to an existing account
or a new one.
'''
import logging
import urllib

from django.contrib.auth import login as plain_login
from django.contrib.auth.models import User
from django.contrib.auth.views import login
from django.core.urlresolvers import reverse
from django.db.utils import IntegrityError
from django.forms.util import ErrorList
from django.http import HttpResponseRedirect
from django.http.response import HttpResponseServerError
from django.shortcuts import render
from django.utils.decorators import method_decorator
from django.views.generic import View
from django_authopenid.forms import OpenidSigninForm, UserAssociation
from django_authopenid.utils import from_openid_response
from django_authopenid.utils import get_url_host
from django_authopenid.views import ask_openid

from djcharme.charme_security_model import LoginForm, UserForm
from djcharme.models import UserProfile
from djcharme.settings import REDIRECT_FIELD_NAME, SEND_MAIL
from djcharme.views import get_extra_context, get_safe_redirect, \
    not_authenticated


LOGGING = logging.getLogger(__name__)

LOGIN_TEMPLATE = 'registration/login.html'
REGISTER_TEMPLATE = 'authopenid/complete.html'


class Login(View):
    """
    Display the login view.

    """
    @method_decorator(not_authenticated)
    def dispatch(self, *args, **kwargs):
        try:
            return super(Login, self).dispatch(*args, **kwargs)
        except Exception as ex:
            LOGGING.error("Login - unexpected error: %s", ex)
            return HttpResponseServerError(str(ex))

    def get(self, request, *args, **kwargs):
        redirect_to = get_safe_redirect(request)
        return login(request, template_name=LOGIN_TEMPLATE,
                     authentication_form=LoginForm,
                     extra_context={REDIRECT_FIELD_NAME:redirect_to})

    def post(self, request, *args, **kwargs):
        auth_form = LoginForm
        extra_context = get_extra_context(kwargs)

        # local sign in
        if 'openid_url' not in request.POST.keys():
            return login(request, template_name=LOGIN_TEMPLATE,
                         authentication_form=auth_form,
                         extra_context=extra_context)

        # open id sign in
        redirect_to = get_safe_redirect(request)
        openid_form = OpenidSigninForm(data=request.POST)
        context = {
            'openid_form': openid_form,
            'form': auth_form(),
            REDIRECT_FIELD_NAME: redirect_to,
        }
        if extra_context is not None:
            context.update(extra_context)

        if openid_form.is_valid():
            redirect_url = ("%s%s?%s" % 
                            (get_url_host(request),
                             reverse('user_complete_signin'),
                             urllib.urlencode
                             ({REDIRECT_FIELD_NAME: redirect_to})))
            return ask_openid(request,
                              openid_form.cleaned_data['openid_url'],
                              redirect_url,
                              on_failure=signin_failure)

        # form not valid
        return render(request, LOGIN_TEMPLATE, context)


class Register(View):
    """
    Display the associate view. This prompts a user to link their openid with a
    local account.

    """
    @method_decorator(not_authenticated)
    def dispatch(self, *args, **kwargs):
        try:
            return super(Register, self).dispatch(*args, **kwargs)
        except Exception as ex:
            LOGGING.error("Register - unexpected error: %s", ex)
            return HttpResponseServerError(str(ex))

    def get(self, request, *args, **kwargs):
        extra_context = get_extra_context(kwargs)
        redirect_to = get_safe_redirect(request)
        openid_ = request.session.get('openid', None)
        if openid_ is None or not openid_:
            # no open id credentials so back to login page
            return HttpResponseRedirect("%s?%s" % 
                                        (reverse('login'),
                                         urllib.urlencode
                                         ({REDIRECT_FIELD_NAME: redirect_to})))

        register_form = UserForm()
        auth_form = LoginForm()
        context = {
            'user_form': register_form,
            'auth_form': auth_form,
            'openid': True,
            REDIRECT_FIELD_NAME: redirect_to,
        }
        if extra_context is not None:
            context.update(extra_context)

        return render(request, REGISTER_TEMPLATE, context)

    def post(self, request, *args, **kwargs):
        extra_context = get_extra_context(kwargs)
        redirect_to = get_safe_redirect(request)
        openid_ = request.session.get('openid', None)
        if openid_ is None or not openid_:
            # no open id credentials so back to login page
            return HttpResponseRedirect("%s?%s" % 
                                        (reverse('user_signin'),
                                         urllib.urlencode
                                         ({REDIRECT_FIELD_NAME: redirect_to})))

        user_ = None
        user_form = UserForm()
        auth_form = LoginForm()

        if 'email' in request.POST.keys():
            user_form = UserForm(request.POST)
            if user_form.is_valid():
                try:
                    user_ = User.objects.create_user(
                        user_form.cleaned_data.get('username'),
                        user_form.cleaned_data.get('email'),
                        password=user_form.cleaned_data.get('password'),
                        first_name=user_form.cleaned_data.get('first_name'),
                        last_name=user_form.cleaned_data.get('last_name'))
                    user_.save()
                    user_profile = UserProfile.objects.create(
                        user_id=user_.id,
                        show_email=user_form.cleaned_data.get('show_email'))
                    user_profile.save()
                except IntegrityError:
                    LOGGING.debug('Username is already registered')
                    errors = user_form._errors.setdefault('username',
                                                          ErrorList())
                    errors.append(u'Username is already registered')
        else:
            auth_form = LoginForm(data=request.POST)
            if auth_form.is_valid():
                user_ = auth_form.get_user()

        if user_ is not None:
            # associate the user to openid
            uassoc = UserAssociation(openid_url=str(openid_),
                                     user_id=user_.id)
            uassoc.save(send_email=SEND_MAIL)
            login(request, user_)
            return HttpResponseRedirect(redirect_to)

        # form not valid
        context = {
            'user_form': user_form,
            'auth_form': auth_form,
            'openid': True,
            REDIRECT_FIELD_NAME: redirect_to,
        }
        if extra_context is not None:
            context.update(extra_context)

        return render(request, REGISTER_TEMPLATE, context)


def signin_success(request, identity_url, openid_response,
        redirect_field_name=REDIRECT_FIELD_NAME, **kwargs):
    """
    openid signin success.

    If the openid is already registered, the user is redirected to 
    url set par next or in settings with OPENID_REDIRECT_NEXT variable.
    If none of these urls are set user is redirectd to /.

    if openid isn't registered user is redirected to register page.
    """
    redirect_to = get_safe_redirect(request)
    openid_ = from_openid_response(openid_response)
    
    openids = request.session.get('openids', [])
    openids.append(openid_)
    request.session['openids'] = openids
    request.session['openid'] = openid_
    try:
        rel = UserAssociation.objects.get(openid_url__exact=str(openid_))
    except:
        # try to register this new user
        return HttpResponseRedirect(
            "%s?%s" % (reverse('user_register'),
            urllib.urlencode({ redirect_field_name: redirect_to }))
        )
    user_ = rel.user
    if user_.is_active:
        user_.backend = "django.contrib.auth.backends.ModelBackend"
        plain_login(request, user_)
    return HttpResponseRedirect(redirect_to)


def signin_failure(request, message, extra_context=None, **kwargs):
    """
    failure with openid signin. Go back to signin page.

    :attr request: request object
    :attr message: an error message
    :attr extra_context: A dictionary of variables to add to the template
    context. Any callable object in this dictionary will be called to produce
    the end result which appears in the context.
    """
    redirect_to = get_safe_redirect(request)
    extra_context = get_extra_context(kwargs)
    openid_form = OpenidSigninForm()
    if message is not None and message is not "":
        openid_form.non_field_errors = openid_form.error_class([message])
    context = {
        'openid_form': openid_form,
        'form': LoginForm(),
        REDIRECT_FIELD_NAME: redirect_to,
    }
    if extra_context is not None:
        context.update(extra_context)
    return render(request, LOGIN_TEMPLATE, context)




'''
BSD Licence
Copyright (c) 2015, Science & Technology Facilities Council (STFC)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
    * Neither the name of the Science & Technology Facilities Council (STFC)
        nor the names of its contributors may be used to endorse or promote
        products derived from this software without specific prior written
        permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


Contents:
This module contains pre-formatted queries of the models.

'''
import logging

from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from provider.oauth2.models import Client

from djcharme.models import FollowedResource, Organization, \
    OrganizationClient, OrganizationUser


LOGGING = logging.getLogger(__name__)

def get_admin_email_addresses(organization_name):
    """
    Get the list of admin email addresses associated with the organization.

    Args:
        organization_name (str): The name of the organization.

    Returns:
        [str] List of admin email addresses.

    """
    organizations = (Organization.objects.filter(name=organization_name))
    if len(organizations) < 1:
        LOGGING.warn("No data found for %s", organization_name)
        return []
    organization_ids = []
    for organization in organizations:
        organization_ids.append(organization.id)
    # there should only be one
    organization_id = organization_ids[0]
    organization_users = (OrganizationUser.objects.filter(role='admin').
                          filter(organization=organization_id))
    if len(organization_users) < 1:
        LOGGING.error("No admin found for organization %s", organization_name)
        return []
    admins = []
    for organization_user in organization_users:
        users = User.objects.filter(username=organization_user.user)
        for user in users:
            admins.append(user.email)
    return admins


def get_users_admin_role_orgs(user_id):
    """
    Get the list of organizations that this user is the admin for.

    Args:
        user_id (str): The id of the user.

    Returns:
        [str] List of organizations that this user is the admin for.

    """
    organization_users = (OrganizationUser.objects.filter(user=user_id).
                          filter(role='admin'))
    organizations = []
    for organization_user in organization_users:
        organizations.append(organization_user.organization.name)
    return organizations


def get_client(organization_name):
    """
    Get the client with the given organization name. Return None if not found.

    Args:
        organization_name (str): The name of the organization.

    Returns:
        client The client object or None if not found.

    """
    try:
        organization = Organization.objects.get(name=organization_name)
        organization_client = OrganizationClient.objects.get(organization=organization)
        client = Client.objects.get(organizationclient=organization_client)
    except ObjectDoesNotExist():
            return None
    return client


def get_user(user_name):
    """
    Get the user with the given user name. Return None if not found.

    Args:
        user_name (str): The user name of the user.

    Returns:
        User The User object or None if not found.

    """
    try:
        user = User.objects.get(username=user_name)
    except ObjectDoesNotExist():
        return None
    except Exception:
        return None
    return user


def get_organization_for_client(client):
    """
    Get the name of the organization associated with the client.

    Args:
        client (client): The Client object from a request

    Returns:
        str The name of the organization or None.

    """
    organization_id = (client.organizationclient_set.
                       values_list('organization', flat=True))
    if len(organization_id) < 1:
        LOGGING.warn("No organization found for client %s", client.url)
        return None
    # there should only be one
    organization_id = organization_id[0]
    organizations = (Organization.objects.filter(id=organization_id))
    for organization in organizations:
        return organization.name
    return None


def get_followers(resources):
    """
    Get the users that are following the resources.

    Args:
        resources(list[str]): The list of URIs of the resources.

    Returns:
        [User] List of users that are following the resource.

    """
    followers = []
    for resource in resources:
        followed_resources = (FollowedResource.objects.
                              filter(resource=resource))
        for followed_resource in followed_resources:
            followers.append(followed_resource)
    return followers


def is_following_resource(user, resource):
    """
    Check if the user is following the resource.

    Args:
        user(User): The name of the user
        resource(str): The URI of the resource.

    Returns:
        True if the user is already following the resource.

    """
    followed_resources = (FollowedResource.objects.
                          filter(user=user).
                          filter(resource=resource))
    if len(followed_resources) > 0:
        return True
    else:
        return False


def is_moderator(user):
    """
    Check to see if this user is in the moderator group. This is the super group
    with privileges to moderate ALL annotations.

    Args:
        user(User): The user object

    Returns:
        boolean True if the user is listed as a member of the moderator group.

    """
    groups = user.groups.values_list('name', flat=True)
    if "moderator" in groups:
        LOGGING.debug("User %s is a moderator", user.username)
        return True
    return False


def is_organization_admin(client, user, annotation_uri):
    """
    Check to see if this user is an admin for the organization at which the
    annotation was created.

    Args:
        client(Client): The client object
        user(User): The user object
        annotation_uri (URIRef): The URI of the annotation.

    Returns:
        boolean True if the user is listed as a admin for the organization.

    """
    user_id = user.id
    organization_id = (client.organizationclient_set.
                       values_list('organization', flat=True))
    if len(organization_id) < 1:
        LOGGING.warn("No organization found for client %s", client.url)
        return False
    # there should only be one
    organization_id = organization_id[0]
    organization_users = (OrganizationUser.objects.filter(user=user_id).
                          filter(organization=organization_id))
    for organization_user in organization_users:
        if organization_user.role == 'admin':
            LOGGING.debug("User %s is an admin for this annotation: %s",
                          user.username, annotation_uri)
            return True
    return False




'''
Created on 18 Nov 2013

@author: mnagni
'''
from httplib import HTTPConnection

from djcharme import settings, HTTP_PROXY, HTTP_PROXY_PORT
from djcharme.node.actions import insert_rdf
from djcharme.node.constants import SUBMITTED


def get_document(source, headers={}, host='data.datacite.org', proxy=None,
                 proxy_port=None, params=None):
    conn = HTTPConnection(host=host)
    if proxy and proxy_port:
        conn = HTTPConnection(proxy, proxy_port)
    # conn.connect()

    request_url = source

    if proxy and proxy_port:
        request_url = 'http://%s/%s' % (host, source)

    if request_url[-1] == '/':
        request_url = request_url[:len(request_url) - 1]
    if params:
        request_url = '%s?%s' % (request_url, params)

    conn.request('GET', request_url, headers=headers)
    res = conn.getresponse()
    if res.status in (302, 303):
        conn.request('GET', res.msg['location'], headers=headers)
        res = conn.getresponse()
    doc = res.read()
    conn.close()
    return doc


def load_doi(doi, graph, user):
    try:
        _id = str(doi)[str(doi).index('//') + 2:]

        response = get_document(_id[_id.index('/') + 1:].encode("utf8"),
                                headers={'accept': 'application/rdf+xml'},
                                host="dx.doi.org",
                                proxy=getattr(settings, HTTP_PROXY),
                                proxy_port=getattr(settings, HTTP_PROXY_PORT))

        insert_rdf(response, 'xml', user,
                   graph=SUBMITTED).serialize(format='turtle')
    except Exception as ex:
        # pass
        print ex



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djcharme', '0002_remove_organizationuser_following'),
    ]

    operations = [
        migrations.AddField(
            model_name='organizationuser',
            name='following',
            field=models.BooleanField(default=False, help_text=b'Send emails about any annotation created or altered by the organization.', verbose_name=b'email changes'),
            preserve_default=True,
        ),
    ]



'''
BSD Licence
Copyright (c) 2015, Science & Technology Facilities Council (STFC)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
    * Neither the name of the Science & Technology Facilities Council (STFC)
        nor the names of its contributors may be used to endorse or promote
        products derived from this software without specific prior written
        permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Created on 12 Apr 2013

@author: mnagni
'''
import base64
from datetime import datetime
import logging
from urllib2 import URLError
import uuid

from SPARQLWrapper.SPARQLExceptions import EndPointNotFound
from django.conf import settings
from django.core.mail import send_mail
from django.db.models import ObjectDoesNotExist
from rdflib import BNode
from rdflib import Graph, URIRef, Literal
from rdflib.graph import ConjunctiveGraph

from djcharme.charme_middleware import CharmeMiddleware
from djcharme.exception import NotFoundError
from djcharme.exception import ParseError
from djcharme.exception import SecurityError
from djcharme.exception import StoreConnectionError
from djcharme.exception import UserError
from djcharme.node.constants import ANNO_URI, LOCALHOST_URI, TARGET_URI, \
    REPLACEMENT_URIS, REPLACEMENT_URIS_MULTIVALUED
from djcharme.node.constants import FOAF, RDF, PROV, OA, CH_NODE
from djcharme.node.constants import FORMAT_MAP, ALLOWED_CREATE_TARGET_TYPE, \
    RESOURCE
from djcharme.node.constants import GRAPH_NAMES, SUBMITTED, INVALID, RETIRED
from djcharme.node.model_queries import get_admin_email_addresses, \
    get_followers, get_organization_for_client, is_moderator, \
    is_organization_admin
from djcharme.node.triple_queries import extract_subject


LOGGING = logging.getLogger(__name__)
CREATED = 'created'

def rdf_format_from_mime(mimetype):
    """
    Get the RDF type based on the mime type.

    Args:
        mimetype (str): The mime type

    Return:
        a string containing the RDF type associated with the mime type.

    """
    for key, value in FORMAT_MAP.iteritems():
        if mimetype == value:
            return key


def format_graph_iri(graph, baseurl='http://dummyhost'):
    '''
        Builds a named graph URIRef using, if exists,
        a settings.SPARQL_QUERY parameter.

        - string **graph**
            the graph name
        * return String
    '''
    if ('http://' in graph) or ('https://' in graph):
        return graph

    return '%s/%s' % (getattr(settings, 'SPARQL_DATA', baseurl), graph)


def generate_graph(store, graph):
    '''
        Generate a new Graph
        - string **graph**
            the graph name
        * return:rdflib.Graph - Returns an RDFlib graph containing the given
                                data
    '''
    return Graph(store=store, identifier=format_graph_iri(graph))


def get_vocab():
    """
        Returns a graph containing all the vocab triples
    """
    return generate_graph(CharmeMiddleware.get_store(), "vocab")


def report_to_moderator(request, resource_id):
    """
    Report this annotation to the moderator of the site where it was created.

    Args:
        request (WSGIRequest): The incoming request.
        resource_id(str): The id of the resource.

    """
    LOGGING.debug("report_to_moderator(request, %s)", resource_id)
    annotation_uri = format_resource_uri_ref(resource_id)
    graph_name = find_annotation_graph_name(annotation_uri)
    graph = generate_graph(CharmeMiddleware.get_store(), graph_name)
    organizations = _get_organization(graph, annotation_uri)
    for res in organizations:
        if res[1] == (URIRef(FOAF + 'name')):
            organization_name = res[2]
            break
    email_addresses = get_admin_email_addresses(organization_name)

    # create message
    message = ('You are receiving this email as you are registered as an ' \
               'admin for %s by the CHARMe site %s\n\nThe annotation\n%s\n' \
               'has been flagged for moderation by ' % 
               (organization_name, getattr(settings, 'NODE_URI'),
                annotation_uri))
    if request.user.first_name != "":
        message = "%s%s " % (message, request.user.first_name)
    if request.user.last_name != "":
        message = "%s%s" % (message, request.user.last_name)
    if request.user.first_name == "" and request.user.last_name == "":
        message = "%suser: %s" % (message, request.user.username)
    if request.user.email != "":
        message = "%s, email: %s " % (message, request.user.email)

    # add reason for flagging annotation
    if request.body != "":
        message = "%s\n\nThe following reason was given:\n%s\n" % (message,
                                                                   request.body)

    # add signature
    message = '%s\n\nRegards\nThe CHARMe site team\n' % (message)

    # send mails
    from_address = getattr(settings, 'DEFAULT_FROM_EMAIL')
    for address in email_addresses:
        send_mail('An annotation has been flagged for moderation',
                  message, from_address, [address])
    return


def insert_rdf(data, mimetype, user, client, graph=None, store=None):
    '''
        Inserts an RDF/json-ld document into the triplestore
        - string **data**
            a document
        - string **mimetype**
            the document mimetype
        - User **user**
            the User object from a request
        - Client **client**
            the Client object from a request
        - string **graph**
            the graph name
        - rdflib.Store **store**
            if none use the return of get_store()
        * return:str - The URI of the new annotation
    '''
    LOGGING.debug("insert_rdf(%s, %s, %s, client, graph, store)", data,
                  mimetype, user)
    try:
        return _insert_rdf(data, mimetype, user, client, graph, store)
    except ParseError as ex:
        raise ParseError(str(ex))
    except EndPointNotFound as ex:
        LOGGING.error("EndPointNotFound error while inserting triple. %s", ex)
        raise StoreConnectionError("Cannot insert triple. " + str(ex))


def _insert_rdf(data, mimetype, user, client, graph, store):
    '''
        Inserts an RDF/json-ld document into the triplestore
        - string **data**
            a document
        - string **mimetype**
            the document mimetype
        - User **user**
            the User object from a request
        - Client **client**
            the Client object from a request
        - string **graph**
            the graph name
        - rdflib.Store **store**
            if none use the return of get_store()
        * return:str - The URI of the new annotation
    '''
    if store is None:
        store = CharmeMiddleware.get_store()
    tmp_g = Graph()
    # Necessary as RDFlib does not contain the json-ld lib
    try:
        tmp_g.parse(data=data, format=mimetype)
    except SyntaxError as ex:
        try:
            LOGGING.info("Syntax error while parsing triple. %s", ex)
            raise ParseError(str(ex))
        except UnicodeDecodeError:
            raise ParseError(ex.__dict__["_why"])
    _format_submitted_annotation(tmp_g)
    final_g = generate_graph(store, graph)

    for nspace in tmp_g.namespaces():
        final_g.store.bind(str(nspace[0]), nspace[1])
    timestamp = Literal(datetime.utcnow())

    # add the person
    person_uri, triples = _create_person(user)
    for triple in triples:
        _add(final_g, triple)

    # add the rest
    anno_uri = ''
    targets = []
    for res in tmp_g:
        if (res[1] == URIRef(RDF + 'type')
            and res[2] == URIRef(OA + 'Annotation')):
            anno_uri = res[0]
            prov = _get_prov(anno_uri, person_uri, client, timestamp)
            for triple in prov:
                _add(final_g, triple)
        if res[1] == URIRef(OA + 'hasTarget'):
            targets.append(res[2])
        _add(final_g, res)
    if targets:
        _mail_followers(anno_uri, targets, CREATED)
    return anno_uri


def modify_rdf(request, mimetype):
    """
    Modify an annotation in the triplestore.

    Args:
        request (WSGIRequest): The http request
        mimetype (str): The document mimetype

    Return:
        a URIRef containing the URI of the modified annotation

    """
    LOGGING.debug("modify_rdf(request, %s)", mimetype)
    try:
        return _modify_rdf(request, mimetype)
    except UserError as ex:
        raise UserError(str(ex))
    except ParseError as ex:
        raise ParseError(str(ex))
    except EndPointNotFound as ex:
        LOGGING.error("EndPointNotFound error while modifying triple. %s", ex)
        raise StoreConnectionError("Cannot insert triple. " + str(ex))


def _modify_rdf(request, mimetype):
    """
    Modify an annotation in the triplestore.

    Args:
        request (WSGIRequest): The http request
        mimetype (str): The document mimetype

    Return:
        a URIRef containing the URI of the modified annotation

    """
    modification_time = Literal(datetime.utcnow())
    store = CharmeMiddleware.get_store()
    tmp_g = Graph()
    data = request.body
    # Necessary as RDFlib does not contain the json-ld lib
    try:
        tmp_g.parse(data=data, format=mimetype)
    except SyntaxError as ex:
        try:
            raise ParseError(str(ex))
        except UnicodeDecodeError:
            raise ParseError(ex.__dict__["_why"])

    original_uri = _get_annotation_uri_from_graph(tmp_g)
    # replace original uri in tmp graph
    for res in tmp_g:
        if res[0] == original_uri:
            tmp_g.remove(res)
            new_res = (URIRef('%s:%s' % (CH_NODE, ANNO_URI)), res[1], res[2])
            tmp_g.add(new_res)

    activity_uri = URIRef((getattr(settings, 'NODE_URI', LOCALHOST_URI)
                  + '/%s/%s' % (RESOURCE, uuid.uuid4().hex)))
    
    original_graph_name = find_annotation_graph_name(original_uri)
    if _is_graph_final_state(original_graph_name):
        raise UserError(("Current annotation status of %s is final. Updates " \
                         "are not allowed." % original_graph_name))
    _format_submitted_annotation(tmp_g)
    final_g = generate_graph(store, SUBMITTED)

    for nspace in tmp_g.namespaces():
        final_g.store.bind(str(nspace[0]), nspace[1])

    # add the person
    person_uri, triples = _create_person(request.user)
    for triple in triples:
        _add(final_g, triple)

    # add the rest
    anno_uri = ''
    targets = []
    for res in tmp_g:
        if (res[1] == URIRef(RDF + 'type')
            and res[2] == URIRef(OA + 'Annotation')):
            anno_uri = res[0]
            prov = _get_prov(anno_uri, person_uri, request.client,
                             modification_time)
            for triple in prov:
                _add(final_g, triple)
            modify_activity = _get_modify_activity(anno_uri, original_uri,
                                                   modification_time,
                                                   activity_uri, person_uri)
            for triple in modify_activity:
                _add(final_g, triple)
        if res[1] == URIRef(OA + 'hasTarget'):
            targets.append(res[2])
        _add(final_g, res)
        
    # retire original
    _change_annotation_state(original_uri, RETIRED, original_graph_name,
                             request, activity_uri, modification_time)

    # send out notifications
    if targets:
        _mail_followers(anno_uri, targets, CREATED, original_uri)
    return anno_uri


def _get_annotation_uri_from_graph(graph):
    """
    Get the URI of the annotation from the given graph.

    Args:
        graph (rdflib.graph.Graph): The graph containing the annotation

    Return:
        a URIRef containing the URI of the annotation

    """
    for res in graph:
        if (res[1] == URIRef(RDF + 'type')
            and res[2] == URIRef(OA + 'Annotation')):
            return res[0]


def _get_prov(annotation_uri, person_uri, client, timestamp):
    """
    Get the provenance data for the annotation.

    Args:
        annotation_uri (URIRef): The URI of the annotation
        person_uri (URIRef): The URI of a person
        client (client): The Client object from a request
        timestamp (Literal(datetime)) The time the annotation was updated
    Returns:
        a list of triples

    """
    triples = []
    triples.append((annotation_uri, URIRef(OA + 'annotatedAt'),
                    timestamp))
    triples.append((annotation_uri, URIRef(OA + 'annotatedBy'), person_uri))
    organization = get_organization_for_client(client)
    if organization != None:
        triples.append((annotation_uri, URIRef(OA + 'annotatedBy'),
                        URIRef(client.url)))
        triples.append((URIRef(client.url), URIRef(RDF + 'type'),
                        URIRef(FOAF + 'Organization')))
        triples.append((URIRef(client.url), URIRef(FOAF + 'name'),
                        Literal(organization)))

    return triples


def _create_person(user):
    """
    Create a persons triples.

    Args:
        user (User): The user details.
    Returns:
        a URIRef containing the person URI
        a list of triples

    """
    person_uri = URIRef((getattr(settings, 'NODE_URI', LOCALHOST_URI)
                  + '/%s/%s' % (RESOURCE, uuid.uuid4().hex)))
    triples = []
    triples.append((person_uri, URIRef(RDF + 'type'),
                    URIRef(FOAF + 'Person')))
    triples.append((person_uri, URIRef(FOAF + 'accountName'),
                    Literal(user.username)))
    if user.last_name != None and len(user.last_name) > 0:
        triples.append((person_uri, URIRef(FOAF + 'familyName'),
                        Literal(user.last_name)))
    if user.first_name != None and len(user.first_name) > 0:
        triples.append((person_uri, URIRef(FOAF + 'givenName'),
                        Literal(user.first_name)))
    try:
        show_email = user.userprofile.show_email
    except ObjectDoesNotExist:
        show_email = False
    if show_email and user.email != None and len(user.email) > 0:
        triples.append((person_uri, URIRef(FOAF + 'mbox'), Literal(user.email)))

    return (person_uri, triples)



def _get_modify_activity(annotation_uri, original_anno_uri, timestamp,
                     activity_uri, person_uri):
    """
    Get the triples for a modify activity.

    Args:
        annotation_uri (URIRef): The URI of the annotation
        original_anno_uri (URIRef): The uri of the original annotation.
        timestamp (Literal): The time of the activity
        activity_uri (URIRef): The URI of the Activity
        person_uri (URIRef): The URI of the person

    Returns:
        a list of triples.

    """
    triples = []
    triples.append((annotation_uri, URIRef(PROV + 'wasGeneratedBy'),
                    activity_uri))
    triples.append((annotation_uri, URIRef(PROV + 'wasRevisionOf'),
                    original_anno_uri))

    triples.append((activity_uri, URIRef(RDF + 'type'),
                    URIRef(PROV + 'Activity')))
    triples.append((activity_uri, URIRef(PROV + 'invalidated'),
                    original_anno_uri))
    triples.append((activity_uri, URIRef(PROV + 'generated'), annotation_uri))

    triples.append((activity_uri, URIRef(PROV + 'wasStartedAt'), timestamp))
    triples.append((activity_uri, URIRef(PROV + 'wasStartedBy'), person_uri))
    triples.append((activity_uri, URIRef(PROV + 'wasEndedAt'), timestamp))
    triples.append((activity_uri, URIRef(PROV + 'wasEndedBy'), person_uri))
    return triples


def _get_deleted_activity(annotation_uri, timestamp, activity_uri, person_uri):
    """
    Get the triples for a delete activity.

    Args:
        annotation_uri (URIRef): The uri of the annotation.
        timestamp (Literal): The time of the activity
        activity_uri (URIRef): The URI of the Activity
        person_uri (URIRef): The URI of the person

    Returns:
        a list of triples.

    """
    triples = []
    triples.append((activity_uri, URIRef(RDF + 'type'),
                    URIRef(PROV + 'Activity')))
    triples.append((activity_uri, URIRef(PROV + 'invalidated'),
                    URIRef(annotation_uri)))
    triples.append((activity_uri, URIRef(PROV + 'wasStartedAt'), timestamp))
    triples.append((activity_uri, URIRef(PROV + 'wasStartedBy'), person_uri))
    triples.append((activity_uri, URIRef(PROV + 'wasEndedAt'), timestamp))
    triples.append((activity_uri, URIRef(PROV + 'wasEndedBy'), person_uri))
    return triples


def format_resource_uri_ref(resource_id):
    '''
        Returns the URIRef associated with the id for this specific node
    '''
    if resource_id.startswith('http:') or resource_id.startswith('https:'):
        return URIRef(resource_id)
    return URIRef('%s/%s/%s' % (getattr(settings, 'NODE_URI', LOCALHOST_URI),
                                RESOURCE, resource_id))


def _format_node_uri_ref(uriref, generated_uris):
    '''
        Rewrite a URIRef according to the node configuration
        * uriref:rdflib.URIRef
        * generated_uris:dict of generated URIs
    '''
    if isinstance(uriref, URIRef) and LOCALHOST_URI in uriref:
        uriref = URIRef(uriref.replace(LOCALHOST_URI,
                                       getattr(settings, 'NODE_URI', LOCALHOST_URI)
                                       + '/'))

    if isinstance(uriref, URIRef) and CH_NODE in uriref:
        uriref = URIRef(uriref.replace(CH_NODE + ':',
                                       getattr(settings, 'NODE_URI', LOCALHOST_URI)
                                       + '/'))

    if isinstance(uriref, URIRef):
        for key in generated_uris.keys():
            if key in uriref:
                uriref = URIRef(uriref.replace(key, "%s/%s" % 
                                               (RESOURCE, generated_uris[key])))
    return uriref


def _format_submitted_annotation(graph):
    '''
        Formats the graph according to the node configuration
    '''
    graph = _validate_submitted_annotation(graph)
    generated_uris = {}
    for id_ in REPLACEMENT_URIS:
        generated_uris[id_] = uuid.uuid4().hex

    target_id_found = False
    target_id_valid = False
    for subject, pred, obj in graph:
        graph.remove((subject, pred, obj))
        # The use of TARGET_URI is only allowed for specific types
        if ((isinstance(subject, URIRef) and TARGET_URI in subject) or
            (isinstance(obj, URIRef) and TARGET_URI in obj)):
            target_id_found = True
            if (pred == URIRef(RDF + 'type') and
                (obj in ALLOWED_CREATE_TARGET_TYPE)):
                target_id_valid = True
        for value in REPLACEMENT_URIS_MULTIVALUED:
            if value in subject:
                bits = subject.split(CH_NODE + ':')
                if len(bits) > 1:
                    key = bits[1]
                    if key not in generated_uris.keys():
                        generated_uris[key] = uuid.uuid4().hex
            if value in obj:
                bits = obj.split(CH_NODE + ':')
                if len(bits) > 1:
                    key = bits[1]
                    if key not in generated_uris.keys():
                        generated_uris[key] = uuid.uuid4().hex

        subject = _format_node_uri_ref(subject, generated_uris)
        pred = _format_node_uri_ref(pred, generated_uris)
        obj = _format_node_uri_ref(obj, generated_uris)
        graph.add((subject, pred, obj))

    if target_id_found and not target_id_valid:
        types = ""
        for type_ in ALLOWED_CREATE_TARGET_TYPE:
            if types != "":
                types = types + ', '
            types = types + type_
        raise UserError((TARGET_URI + ' may only be used for ' + types + 
                         ' target types'))


def _validate_submitted_annotation(graph):
    """
    Validate the graph.

    Args:
        graph (rdflib.graph.Graph): The graph containing an annotation.

    Returns:
        graph (rdflib.graph.Graph): The validated graph.
    """
    local_resource = getattr(settings, 'NODE_URI', LOCALHOST_URI) + '/'
    for subject, _, _ in graph:
        if local_resource in subject:
            LOGGING.info("UserError Found %s in the subject of submitted " \
                         "annotation)", subject)
            raise UserError(str(subject) + " is not allowed as the subject " \
                            "of a triple")
    return graph


def change_annotation_state(annotation_uri, new_graph_name, request):
    """
    Advance the status of an annotation.

    Args:
        annotation_uri (URIRef): The URI of the annotation.
        new_graph_name (str): The name of the graph/state to move the
            annotation to.
        request (WSGIRequest): The incoming request.

    Returns:
        graph (rdflib.graph.Graph): The new graph containing the updated
        annotation.

    """
    LOGGING.debug("change_annotation_state(%s, %s, request)",
                  annotation_uri, new_graph_name)
    validate_graph_name(new_graph_name)
    annotation_uri = format_resource_uri_ref(annotation_uri)
    activity_uri = URIRef((getattr(settings, 'NODE_URI', LOCALHOST_URI)
                           + '/%s/%s' % (RESOURCE, uuid.uuid4().hex)))
    timestamp = Literal(datetime.utcnow())
    old_graph_name = find_annotation_graph_name(annotation_uri)
    new_g = _change_annotation_state(annotation_uri, new_graph_name,
                                     old_graph_name, request, activity_uri,
                                     timestamp, True)
    if new_g == None:
        return

    # add the person
    person_uri, triples = _create_person(request.user)
    for triple in triples:
        try:
            _add(new_g, triple)
        except EndPointNotFound as ex:
            raise StoreConnectionError("Cannot insert triple. " + str(ex))

    # If we are retiring include extra metadata
    if _is_graph_final_state(new_graph_name):
        deleted_prov = _get_deleted_activity(annotation_uri, timestamp,
                                             activity_uri, person_uri)
        for triple in deleted_prov:
            _add(new_g, triple)

    # TODO add extra prov for change to submitted
    # TODO add extra prov for change to stable
    return new_g


def _change_annotation_state(annotation_uri, new_graph_name, old_graph_name, request,
                             activity_uri, timestamp,
                             delete_body_target=False):
    """
    Advance the status of an annotation.

    Args:
        annotation_uri (URIRef): The URI of the annotation.
        new_graph_name (str): The name of the graph/state to move the annotation to.
        old_graph_name (str): The name of the graph/state to move the annotation from.
        request (WSGIRequest): The incoming request.
        activity_uri (URIRef): The uri of the Activity
        timestamp (Literal): The time of the change
        delete_body_target (boolean): Physically delete the target if the
            annotation is moved to the retired or invalid graph

    Returns:
        graph (rdflib.graph.Graph): The new graph containing the updated
        annotation.

    """
    # lets do some initial validation
    annotation_uri = format_resource_uri_ref(annotation_uri)
    if old_graph_name == new_graph_name:
        return None
    if _is_graph_final_state(old_graph_name):
        raise UserError(("Current annotation status of %s is final. Status " \
                         "has not been updated." % old_graph_name))
    if _is_graph_final_state(new_graph_name):
        # we must do this before we move the annotation
        if delete_body_target:
            _delete_target(annotation_uri, old_graph_name, request)
            _delete_body(annotation_uri, old_graph_name, request)
    new_g = _move_annotation(annotation_uri, new_graph_name, old_graph_name, request,
                             timestamp)
    if _is_graph_final_state(new_graph_name):
        _add(new_g, (annotation_uri, URIRef(PROV + 'wasInvalidatedBy'),
                     activity_uri))
    return new_g


def validate_graph_name(graph_name):
    """
    Check that the graph name is valid.

    Args:
        graph_name (str): The graph name to validate

    Throws:
        UserError if graph name is invalid

    """
    if graph_name in GRAPH_NAMES:
        return
    names = ''
    # prepare error message
    for name in GRAPH_NAMES:
        if names != '':
            names = names + ', '
        names = names + name
    raise UserError(("The status of %s is not valid. It must be one of %s" % 
                     (graph_name, names)))


def _move_annotation(annotation_uri, new_graph, old_graph, request, timestamp):
    """
    Move an annotation from one graph to another.

    Args:
        annotation_uri (URIRef): The URI of the annotation.
        new_graph (str): The name of the graph to move the annotation to.
        old_graph (str): The name of the graph to move the annotation from.
        request (WSGIRequest): The incoming request.
        timestamp (Literal): The time of the move

    Returns:
        graph (rdflib.graph.Graph): The new graph containing the updated
        annotation.

    """
    # First check permissions
    old_g = generate_graph(CharmeMiddleware.get_store(), old_graph)
    if not is_update_allowed(old_g, annotation_uri, request):
        raise SecurityError(("You do not have the required permission to " \
                             "update the status of annotation %s" % 
                             annotation_uri))

    new_g = generate_graph(CharmeMiddleware.get_store(), new_graph)
    # Move the people
    for res in _get_people(old_g, annotation_uri):
        _remove(old_g, res)
        _add(new_g, res)
    # Copy the organization
    for res in _get_organization(old_g, annotation_uri):
        _add(new_g, res)
    # Copy the software
    for res in _get_software(old_g, annotation_uri):
        _add(new_g, res)
    # Move the annotation
    targets = []
    for res in old_g.triples((annotation_uri, None, None)):
        _remove(old_g, res)
        # We are only allowed one annotatedAt per annotation
        if res[1] == URIRef(OA + 'annotatedAt'):
            continue
        if res[1] == URIRef(OA + 'hasTarget'):
            targets.append(res[2])
        _add(new_g, res)
    # Add new annotatedAt
    _add(new_g, ((annotation_uri, URIRef(OA + 'annotatedAt'), timestamp)))
    
    if targets:
        if new_graph == RETIRED:
            message = "marked as deleted"
        else:
            message = "marked as %s" % new_graph

        _mail_followers(annotation_uri, targets, message)

    return new_g


def _delete_body(annotation_uri, graph_name, request):
    """
    Delete the bodies associated with an annotation if they are no longer
    referenced and they have external URIs.

    Args:
        annotation_uri (URIRef): The URI of the annotation.
        graph_name (str): The name of the graph to move the annotation from.
        request (WSGIRequest): The incoming request.

    """
    # First check permissions
    graph = generate_graph(CharmeMiddleware.get_store(), graph_name)
    if not is_update_allowed(graph, annotation_uri, request):
        raise SecurityError(("You do not have the required permission to " \
                             "update the status of annotation %s" % 
                             annotation_uri))

    # Find all of the targets
    for res in graph.triples((annotation_uri, OA['hasBody'], None)):
        # If this is the only reference to the body then delete it
        count = 0
        for _ in graph.triples((None, None, res[2])):
            count = count + 1
        if count > 1:
            continue
        node_uri = getattr(settings, 'NODE_URI', LOCALHOST_URI)
        for body in  graph.triples((res[2], None, None)):
            if node_uri in body[0]:
                continue
            LOGGING.debug("permanently deleting body %s", body)
            _remove(graph, body)


def _delete_target(annotation_uri, graph_name, request):
    """
    Delete the targets associated with an annotation if they are no longer
    referenced and they have external URIs.

    Args:
        annotation_uri (URIRef): The URI of the annotation.
        graph_name (str): The name of the graph to move the annotation from.
        request (WSGIRequest): The incoming request.

    """
    # First check permissions
    graph = generate_graph(CharmeMiddleware.get_store(), graph_name)
    if not is_update_allowed(graph, annotation_uri, request):
        raise SecurityError(("You do not have the required permission to " \
                             "update the status of annotation %s" % 
                             annotation_uri))

    # Find all of the targets
    for res in graph.triples((annotation_uri, OA['hasTarget'], None)):
        # If this is the only reference to the target then delete it
        count = 0
        for _ in graph.triples((None, None, res[2])):
            count = count + 1
        if count > 1:
            continue
        node_uri = getattr(settings, 'NODE_URI', LOCALHOST_URI)
        for target in  graph.triples((res[2], None, None)):
            if node_uri in target[0]:
                continue
            LOGGING.debug("permanently deleting target %s", target)
            _remove(graph, target)


def is_update_allowed(graph, annotation_uri, request):
    """
    Check if this user is allowed to update this annotation.

    Args:
        graph (rdflib.graph.Graph): The graph containing the annotation.
        annotation_uri (URIRef): The URI of the annotation.
        request (WSGIRequest): The incoming request.

    Returns:
        boolean True if the user is allowed to update this annotation.

    """
    if _is_my_annotation(graph, annotation_uri, request.user.username):
        return True
    try:
        if is_organization_admin(request.client, request.user, annotation_uri):
            return True
    except AttributeError:
        # there is no request.client
        pass
    if is_moderator(request.user):
        return True

    return False


def _is_my_annotation(graph, annotation_uri, username):
    """
    Check to see if this annotation was edited by the user.

    Args:
        graph (rdflib.graph.Graph): The graph containing the annotation.
        annotation_uri (URIRef): The URI of the annotation.
        userername (str): The name of the user to check

    Returns:
        boolean True if the user is listed as an accountName in the annotatedBy
        Person object of the annotation.

    """
    for res in graph.triples((annotation_uri, URIRef(OA + 'annotatedBy'),
                              None)):
        for res2 in graph.triples((res[2], URIRef(FOAF + 'accountName'), None)):
            if str(res2[2]) == username:
                LOGGING.debug("User %s is the owner of this annotation %s",
                              username, annotation_uri)
                return True
    return False


def _get_people(graph, annotation_uri):
    """
    Get the list of people associated with the annotation in the graph.

    Args:
        graph (rdflib.graph.Graph): The graph containing the annotation.
        annotation_uri (URIRef): The URI of the annotation.

    Returns:
        list[tuple] The list of people associated with the annotation.

    """
    people = []
    for res in graph.triples((annotation_uri, URIRef(OA + 'annotatedBy'),
                              None)):
        for res2 in graph.triples((res[2], URIRef(RDF + 'type'),
                                   URIRef(FOAF + 'Person'))):
            for res3 in graph.triples((res2[0], None, None)):
                people.append(res3)
    return people


def _get_organization(graph, annotation_uri):
    """
    Get the list of organizations associated with the annotation in the graph.

    Args:
        graph (rdflib.graph.Graph): The graph containing the annotation.
        annotation_uri (URIRef): The URI of the annotation.

    Returns:
        list[tuple] The list of organizations associated with the annotation.

    """
    organization = []
    for res in graph.triples((annotation_uri, URIRef(OA + 'annotatedBy'),
                              None)):
        for res2 in graph.triples((res[2], URIRef(RDF + 'type'),
                                   URIRef(FOAF + 'Organization'))):
            for res3 in graph.triples((res2[0], None, None)):
                organization.append(res3)
    return organization


def _get_software(graph, annotation_uri):
    """
    Get the list of software associated with the annotation in the graph.

    Args:
        graph (rdflib.graph.Graph): The graph containing the annotation.
        annotation_uri (URIRef): The URI of the annotation.

    Returns:
        list[tuple] The list of software associated with the annotation.

    """
    software = []
    for res in graph.triples((annotation_uri, URIRef(OA + 'serializedBy'),
                              None)):
        for res2 in graph.triples((res[2], URIRef(RDF + 'type'),
                                   URIRef(PROV + 'SoftwareAgent'))):
            for res3 in graph.triples((res2[0], None, None)):
                software.append(res3)
    return software


def _mail_followers(annotation_uri, targets, message_part, original_uri=None):
    """
    Mail the followers of the targets and annotation.

    Args:
        annotation_uri (str): The URI of the annotation.
        targets (list(str)): The list of URIs of targets.
        message_part (str): The action on the annotation.
        original_uri (str): The URI of the original annotation, may be None.

    """
    followers = get_followers(targets)
    from_address = getattr(settings, 'DEFAULT_FROM_EMAIL')
    subject = 'An annotation has been %s' % message_part

    for follower in followers:
        # create message
        message = ('You are receiving this email as you are registered as ' \
                   'following %s by the CHARMe site %s\n\nThe annotation ' \
                   '%s, which references %s, has been %s.\n' % 
                   (follower.resource, getattr(settings, 'NODE_URI'),
                    annotation_uri, follower.resource, message_part))
        if original_uri:
            message = ('%s This is a modification of the annotation %s' % 
                       (message, original_uri))
        message = ('%s\n\nYou can update your notification settings by ' \
                   'logging on to %s\n\nRegards\nThe CHARMe site team\n' % 
                   (message, getattr(settings, 'NODE_URI')))        
        # send mails
        try:
            send_mail(subject, message, from_address, [follower.user.email])
        except Exception as ex:
            LOGGING.error("An error occurred when emailing user %s, %s" % 
                          (follower.user, ex))
    
    if message_part == CREATED:
        # The annotation is being created so there will be no followers
        return
    followers = get_followers([annotation_uri])
    for follower in followers:
        # create message
        message = ('You are receiving this email as you are registered as ' \
                   'following %s by the CHARMe site %s\n\nThis annotation ' \
                   'has been %s. You can update your notification settings ' \
                   'by logging on to %s\n\nRegards\nThe CHARMe site team\n' \
                   % (follower.resource, getattr(settings, 'NODE_URI'),
                      message_part, getattr(settings, 'NODE_URI')))
        # send mails
        try:
            send_mail(subject, message, from_address, [follower.user.email])
        except Exception as ex:
            LOGGING.error("An error occurred when emailing user %s, %s" % 
                          (follower.user, ex))
    return


def find_annotation_graph_name(resource_id):
    """
    Find the name of the graph that contains the given resource.

    Args:
        resource_id(str): The id of the resource.

    Returns:
        str The name of the graph.

    Throws:
        NotFoundError

    """
    triple = (format_resource_uri_ref(resource_id), None, None)
    for graph in GRAPH_NAMES:
        new_g = generate_graph(CharmeMiddleware.get_store(), graph)
        if triple in new_g:
            return graph
    raise NotFoundError(("Annotation %s not found" % resource_id))


def find_resource_by_id(resource_id, depth=1):
    '''
        Returns the charme resource associated with the given resource_id
        * resource_id:String
        * return: an rdflib.Graph object
    '''
    graph = ConjunctiveGraph(store=CharmeMiddleware.get_store())
    uri_ref = format_resource_uri_ref(resource_id)
    LOGGING.debug("Looking for resource %s", uri_ref)
    return extract_subject(graph, uri_ref, depth)


def resource_exists(resource_id):
    """
    Check that the resource exists in the triple store.

    Args:
        resource_id(str): The id of the resource.

    Returns:
        True if the resource exists.

    """
    graph = find_resource_by_id(resource_id)
    if len(graph) > 0:
        return True
    return False


# This code is a workaround until FUSEKI fixes this bug
# https://issues.apache.org/jira/browse/JENA-592
def __query_annotations(graph, default_graph, pred=None, obj=None):
    query = ''
    if obj:
        query = '''
            SELECT ?subject ?pred ?obj WHERE { GRAPH <%s> {?subject ?pred <%s> }}
        ''' % (default_graph, obj)
    if pred:
        query = '''
            SELECT ?subject ?pred ?obj WHERE { GRAPH <%s> {?subject <%s> ?obj }}
        ''' % (default_graph, pred)
    return graph.query(query)


def collect_annotations(graph_name):
    '''
        Returns a graph containing all the node annotations
        - string **graph_name**
            the graph name
    '''
    graph = generate_graph(CharmeMiddleware.get_store(), graph_name)
    tmp_g = Graph()

    anno = graph.triples((None, None, OA['Annotation']))
    target = graph.triples((None, OA['hasTarget'], None))
    body = graph.triples((None, OA['hasBody'], None))

    try:
        for res in anno:
            tmp_g.add(res)
        for res in target:
            tmp_g.add(res)
        for res in body:
            tmp_g.add(res)
    except URLError as ex:
        raise StoreConnectionError("Cannot open a connection with triple store"
                                   "\n" + str(ex))
    except EndPointNotFound as ex:
        raise StoreConnectionError("Cannot open a connection with triple store"
                                   "\n" + str(ex))
    return tmp_g


def _is_graph_final_state(graph_name):
    """
    Check if the graph permits modifications to annotations.

    Args:
        graph_name(str): The name of the graph.

    Returns:
        True if the graph permits modifications to annotations.

    """
    if graph_name == INVALID or graph_name == RETIRED:
        return True
    return False


def _add(graph, spo):
    """
    Add a triple to the graph.

    """
    if getattr(settings, 'STRABON', False):
        _add_strabon(spo, graph)
    else:
        graph.add(spo)


def _add_strabon(spo, context=None):
    """
    Add a triple to the store of triples.

    This is an upated version of the add method from sparqlstore for use with
    strabon.

    """
    from rdflib.plugins.stores.sparqlstore import SPARQLUpdateStore
    sparqlstore = SPARQLUpdateStore(queryEndpoint=getattr(settings,
                                                          'SPARQL_QUERY'),
                                    update_endpoint=getattr(settings,
                                                            'SPARQL_UPDATE'),
                                    postAsEncoded=False)
    sparqlstore.bind("chnode", getattr(settings, 'NODE_URI',
                                       'http://localhost'))

    sparqlstore.headers["Content-type"] = "application/x-www-form-urlencoded"
    credentials = "%s:%s" % (getattr(settings, 'SPARQL_USERNAME'),
                                     getattr(settings, 'SPARQL_PASSWORD'))
    credentials64 = base64.b64encode(credentials.encode('utf-8'))
    sparqlstore.headers["Authorization"] = ("Basic %s" % credentials64)

    if not sparqlstore.connection:
        raise "UpdateEndpoint is not set - call 'open'"

    (subject, predicate, obj) = spo
    if (isinstance(subject, BNode) or
        isinstance(predicate, BNode) or
        isinstance(obj, BNode)):
        raise Exception("SPARQLStore does not support Bnodes! See " \
                        "http://www.w3.org/TR/sparql11-query/#BGPsparqlBNodes")

    triple = "%s %s %s " % (subject.n3(), predicate.n3(), obj.n3())
    if context is not None:
        q = "INSERT DATA { GRAPH %s { %s } }" % (
            context.identifier.n3(), triple)
    else:
        q = "INSERT DATA { %s }" % triple
    q = 'view=HTML&format=HTML&handle=plain&submit=Update&query=%s' % q
    r = sparqlstore._do_update(q)
    content = r.read()  # we expect no content
    if r.status not in (200, 204):
        sparqlstore.close()
        raise Exception("Could not update: %d %s\n%s" % (
            r.status, r.reason, content))
    sparqlstore.close()


def _remove(graph, spo):
    """
    Remove a triple from the graph.

    """
#     graph.remove(spo)
    if getattr(settings, 'STRABON', False):
        _remove_strabon(spo, graph)
    else:
        graph.remove(spo)


def _remove_strabon(spo, context):
    """
    Remove a triple from the store.

    This is an upated version of the remove method from sparqlstore for use with
    strabon.

    """
    from rdflib.plugins.stores.sparqlstore import SPARQLUpdateStore
    from rdflib import Variable
    sparqlstore = SPARQLUpdateStore(queryEndpoint=getattr(settings,
                                                          'SPARQL_QUERY'),
                                    update_endpoint=getattr(settings,
                                                            'SPARQL_UPDATE'),
                                    postAsEncoded=False)
    sparqlstore.bind("chnode", getattr(settings, 'NODE_URI',
                                       'http://localhost'))

    sparqlstore.headers["Content-type"] = "application/x-www-form-urlencoded"
    credentials = "%s:%s" % (getattr(settings, 'SPARQL_USERNAME'),
                                     getattr(settings, 'SPARQL_PASSWORD'))
    credentials64 = base64.b64encode(credentials.encode('utf-8'))
    sparqlstore.headers["Authorization"] = ("Basic %s" % credentials64)

    if not sparqlstore.connection:
        raise "UpdateEndpoint is not set - call 'open'"

    (subject, predicate, obj) = spo
    if not subject:
        subject = Variable("S")
    if not predicate:
        predicate = Variable("P")
    if not obj:
        obj = Variable("O")

    triple = "%s %s %s ." % (subject.n3(), predicate.n3(), obj.n3())
    if context is not None:
        q = "DELETE { GRAPH %s { %s } } WHERE { GRAPH %s { %s } }" % (
            context.identifier.n3(), triple,
            context.identifier.n3(), triple)
    else:
        q = "DELETE { %s } WHERE { %s }" % (triple, triple)
    q = 'view=HTML&format=HTML&handle=plain&submit=Update&query=%s' % q
    r = sparqlstore._do_update(q)
    content = r.read()  # we expect no content
    if r.status not in (200, 204):
        raise Exception("Could not update: %d %s\n%s" % (
            r.status, r.reason, content))
    sparqlstore.close()




'''
BSD Licence
Copyright (c) 2014, Science & Technology Facilities Council (STFC)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
    * Neither the name of the Science & Technology Facilities Council (STFC)
        nor the names of its contributors may be used to endorse or promote
        products derived from this software without specific prior written
        permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
Created on 1 Nov 2011

@author: Maurizio Nagni
'''
import logging
import socket

from django.contrib import messages
from django.contrib.messages.api import MessageFailure
from django.core.context_processors import csrf
from django.http.response import HttpResponse
from django.shortcuts import render_to_response
from django.utils.safestring import mark_safe
from rdflib.plugin import PluginException

from djcharme import mm_render_to_response_error
from djcharme.charme_middleware import CharmeMiddleware
from djcharme.node.constants import FORMAT_MAP
from djcharme.node.search import get_multi_value_parameter_names


LOGGING = logging.getLogger(__name__)


def _build_host_url(request):
    hostname = socket.getfqdn()
    if (request.META['SERVER_PORT'] != str(80)
        and request.META['SERVER_PORT'] != str(443)):
        hostname = "%s:%s" % (hostname, request.META['SERVER_PORT'])
    if request.is_secure():
        return 'https://%s' % (hostname)
    else:
        return 'http://%s' % (hostname)


def get_home(request):
    context = {}
    context['hostURL'] = _build_host_url(request)
    return _dispatch_response(request, 'homeTemplate', context)


def get_description(request, collection_guid=None,
                    observation_guid=None,
                    result_guid=None):
    host_url = _build_host_url(request)
    ospath = _build_description_ospath(host_url, collection_guid,
                                       observation_guid, result_guid)
    response = CharmeMiddleware.get_osengine().get_description(ospath)
    context = {}
    context['response'] = mark_safe(response)
    return _dispatch_response(request, 'responseTemplate.html', context)


def do_search(request, iformat):
    host_url = _build_host_url(request)
    context = _update_context(request)
    try:
        response = CharmeMiddleware.get_osengine().do_search(host_url,
                                                             iformat, context)
        return HttpResponse(response, content_type=FORMAT_MAP.get(iformat))
    except Exception as ex:
        try:
            messages.add_message(request, messages.ERROR, ex)
            LOGGING.error(str(ex))
        except PluginException as ex:
            LOGGING.error(str(ex))
        except MessageFailure as ex:
            LOGGING.error(str(ex))
        return mm_render_to_response_error(request, '503.html', 503)


def do_suggest(request, iformat):
    host_url = _build_host_url(request)
    context = _update_context(request)
    try:
        # TODO change when ceda_markup has been updated to accept do_suggest
        context['suggest'] = True
        response = CharmeMiddleware.get_osengine().do_search(host_url,
                                                             iformat, context)
        return HttpResponse(response, content_type=FORMAT_MAP.get(iformat))
    except Exception as ex:
        try:
            messages.add_message(request, messages.ERROR, ex)
            LOGGING.error(str(ex))
        except PluginException as ex:
            LOGGING.error(str(ex))
        except MessageFailure as ex:
            LOGGING.error(str(ex))
        return mm_render_to_response_error(request, '503.html', 503)


def _update_context(request):
    context = CharmeMiddleware.get_osengine().create_query_dictionary()
    search_parameter_names = get_multi_value_parameter_names()
    if request.GET is not None:
        # TODO remove next two lines
        for param in request.GET.iteritems():
            context[param[0]] = param[1]
            # TODO use list when I can work out how to sort out the response
#         for key in request.GET.keys():
#             if search_parameter_names.count(key) > 0:
#                 context[key] = request.GET.getlist(key)
#             else:
#                 context[key] = request.GET.get(key)
    context.update(context)
    return context


def _build_description_ospath(hostURL, collection_guid=None,
                              observation_guid=None, result_guid=None):
    ospath = "%s/search/" % (hostURL)
    if collection_guid:
        ospath = "%s%s/" % (ospath, collection_guid)
    if observation_guid:
        ospath = "%s%s/" % (ospath, observation_guid)
    if result_guid:
        ospath = "%s%s/" % (ospath, result_guid)
    return ospath


def _dispatch_response(request, template, context):
    context.update(csrf(request))
    return render_to_response(template, context)



"""
Custom models.

"""
from django.conf import settings
from django.contrib.auth.models import User
from django.db import models
from provider.oauth2.models import Client


AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


class FollowedResource(models.Model):
    """
    The resource followed by a user.

    """
    user = models.ForeignKey(User,
                             help_text=
                             'The user to associate with this resource.')
    resource = models.CharField(max_length=500,
                            help_text=
                            "The URI of the resource followed by the user.")

    def __unicode__(self):
        return self.resource


class UserProfile(models.Model):
    """
    Additional information about a user.

    """
    user = models.OneToOneField(User)
    show_email = models.BooleanField('show email',
                                     default=False,
                                     help_text='Include your email address ' \
                                     'in any new annotations you create.')


class Organization(models.Model):
    """
    Information about an organization.

    """
    name = models.CharField(max_length=100, blank=False, unique=True,
                            help_text='The name of the organization.')
    primary_email = models.EmailField(blank=False, help_text=
                                      'The email of a person/list who is ' \
                                      'responsible for the CHAMRe client at ' \
                                      'this organization.')

    def __unicode__(self):
        return self.name


class OrganizationClient(models.Model):
    """
    The association of a client to an organization.

    """
    organization = models.ForeignKey(Organization,
                            help_text='The name of the organization.')
    client = models.ForeignKey(Client, unique=True, help_text=
                               'The client to associate with this ' \
                               'organization.')

    def __unicode__(self):
        return self.organization.name


class OrganizationUser(models.Model):
    """
    The role within an organization for a user.

    """
    organization = models.ForeignKey(Organization,
                                     help_text='The name of the organization.')
    user = models.ForeignKey(User,
                             help_text=
                             'The user to associate with this organization.')
    role = models.CharField(max_length=10,
                            choices=[('user', 'user'), ('admin', 'admin')],
                            help_text=
                            "The user's role within this organization.")

    def __unicode__(self):
        return self.role




'''
BSD Licence
Copyright (c) 2015, Science & Technology Facilities Council (STFC)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
    * Neither the name of the Science & Technology Facilities Council (STFC)
        nor the names of its contributors may be used to endorse or promote
        products derived from this software without specific prior written
        permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


Contents:
This module contains charme security model including login and user profile
forms.

Created on 11 Dec 2013

@author: mnagni
'''
import logging

from django import forms
from django.contrib.auth import authenticate, get_user_model
from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.models import User
from django.core.exceptions import ValidationError
from django.forms.fields import BooleanField, CharField, EmailField
from django.forms.forms import Form
from django.forms.widgets import PasswordInput
from django.utils.text import capfirst

from djcharme.exception import SecurityError
from djcharme.models import UserProfile


LOGGING = logging.getLogger(__name__)


class UserForm(Form):
    username = CharField(max_length=30, required=True)
    first_name = CharField(max_length=30, required=False)
    last_name = CharField(max_length=30, required=False)
    password = CharField(max_length=30, widget=PasswordInput(), required=True)
    confirm_password = CharField(max_length=30, widget=PasswordInput(),
                                 required=True)
    email = EmailField(required=True)
    show_email = BooleanField(required=False)
    conditions_of_use = BooleanField(required=False)

    def clean(self):
        if not self.cleaned_data.get('conditions_of_use'):
            raise ValidationError(
                "You must accept the conditions of use."
            )

        if (self.cleaned_data.get('password')
            != self.cleaned_data.get('confirm_password')):
            raise ValidationError(
                "Passwords must match."
            )
        return self.cleaned_data


class UserUpdateForm(forms.ModelForm):
    class Meta:
        model = User
        fields = ('first_name', 'last_name', 'email')

    def clean(self):
        return self.cleaned_data


class UserProfileUpdateForm(forms.ModelForm):
    class Meta:
        model = UserProfile
        fields = ('show_email',)

    def clean(self):
        return self.cleaned_data

class LoginForm(Form):
    username = CharField(max_length=30, required=True)
    password = CharField(max_length=30,
                         widget=PasswordInput(),
                         required=True)

    error_messages = {
        'invalid_login': ("Please enter a correct username and password. "
                          "Note that both fields may be case-sensitive."),
        'inactive': "This account is inactive."
    }

    def __init__(self, request=None, *args, **kwargs):
        """
        The 'request' parameter is set for custom auth use by subclasses.
        The form data comes in via the standard 'data' kwarg.
        """
        self.request = request
        self.user_cache = None
        super(LoginForm, self).__init__(*args, **kwargs)

        # Set the label for the "username" field.
        user_model = get_user_model()
        self.username_field = (user_model._meta.get_field
                               (user_model.USERNAME_FIELD))
        if self.fields['username'].label is None:
            self.fields['username'].label = (capfirst
                                             (self.username_field.verbose_name))

    def clean(self):
        username = self.cleaned_data.get('username')
        password = self.cleaned_data.get('password')

        if username and password:
            self.user_cache = authenticate(username=username,
                                           password=password)
            if self.user_cache is None:
                raise forms.ValidationError(
                    self.error_messages['invalid_login'],
                    code='invalid_login',
                    params={'username': self.username_field.verbose_name},
                )
            elif not self.user_cache.is_active:
                raise forms.ValidationError(
                    self.error_messages['inactive'],
                    code='inactive',
                )
        return self.cleaned_data

    def check_for_test_cookie(self):
        LOGGING.warn("check_for_test_cookie is deprecated; ensure your login "
                     "view is CSRF-protected.", DeprecationWarning)

    def get_user_id(self):
        if self.user_cache:
            return self.user_cache.id
        return None

    def get_user(self):
        return self.user_cache


class CharmeAuthenticationBackend(ModelBackend):
    """
    Extends Django's ``ModelBackend`` to allow login via username,
    or verification token.

    Args are either ``username`` and ``password``
    and ``token``. In either case, ``is_active`` can also be given.

    For login, is_active is not given, so that the login form can
    raise a specific error for inactive users.
    For password reset, True is given for is_active.
    For signup verficiation, False is given for is_active.
    """

    def __init__(self, *args, **kwargs):
        super(CharmeAuthenticationBackend, self).__init__(*args, **kwargs)

    def authenticate(self, **kwargs):
        if kwargs:
            username = kwargs.pop("username", None)
            password = kwargs.pop("password", None)
            if username:
                try:
                    backend = ModelBackend()
                    return backend.authenticate(username=username,
                                                password=password)
                except Exception:
                    LOGGING.error("Wrong password for username: " +
                                  str(username))
                    raise SecurityError()
